from django.apps import AppConfig

class WagtailWordpressImportImportersConfig(AppConfig):
    label = "wagtail_wordpress_import_importers"
    name = "wagtail_wordpress_import_importers"
    verbose_name = "Wagtail wordpress import importers"



import os

from django.core.management.base import BaseCommand
from prettytable import PrettyTable
from wagtail_wordpress_import.analysis import HTMLAnalyzer
from wagtail_wordpress_import.importers.wordpress import WordpressImporter


class Command(BaseCommand):
    help = """This command is not used directly in an import.

    It's a tool to help you see and understand the html tags in the content as well as the inline styles found.

    It ouputs a series of tables to the command line.
    The ouput is ideally used inside a spreadsheet by piping it out to a file and importing it.
    Hint: Use the | as a column separator when importing

    Use a command like: ./manage.py analyze_html_content [your_source_xml_file] > analysis.txt
    Then remove the titles and horizontal table lines so it looks like:

    |    Tag     | Pages used on | Total occurrences |
    |     p      |      2504     |       41696       |
    |     a      |      2433     |       19646       |
    |   strong   |      2147     |       21020       |
    ...

    and save each table to a separate file for later import.
    """

    def add_arguments(self, parser):
        parser.add_argument("xml_file", type=str, help="The full path to your xml file")
        parser.add_argument(
            "-t",
            "--type",
            type=str,
            help="The wordpress post type/s to import. Use a comma to separate multiple types",
            default="page,post",
        )
        parser.add_argument(
            "-s",
            "--status",
            type=str,
            help="The wordpress post statuse/s to import. Use a comma to separate multiple types",
            default="publish,draft",
        )

    def handle(self, **options):
        xml_file_path = self.get_xml_file(f"{options['xml_file']}")
        importer = WordpressImporter(xml_file_path)
        analyzer = HTMLAnalyzer()

        importer.analyze_html(
            analyzer,
            page_types=options["type"].split(","),
            page_statuses=options["status"].split(","),
        )

        # Tags
        tags_table = PrettyTable()
        tags_table.field_names = ["Tag", "Pages used on", "Total occurrences"]
        for tag, total_pages in analyzer.tags_unique_pages.most_common():
            tags_table.add_row([tag, total_pages, analyzer.tags_total[tag]])

        self.stdout.write("Most commonly used HTML tags")
        self.stdout.write(str(tags_table))

        # Attributes
        attributes_table = PrettyTable()
        attributes_table.field_names = [
            "Tag",
            "Attribute",
            "Pages used on",
            "Total occurrences",
        ]
        for (
            tag,
            attribute,
        ), total_pages in analyzer.attributes_unique_pages.most_common():
            attributes_table.add_row(
                [
                    tag,
                    attribute,
                    total_pages,
                    analyzer.attributes_total[(tag, attribute)],
                ]
            )

        self.stdout.write("")
        self.stdout.write("Most commonly used HTML attributes.")
        self.stdout.write(str(attributes_table))

        # Styles
        styles_table = PrettyTable()
        styles_table.field_names = [
            "Tag",
            "Style",
            "Pages used on",
            "Total occurrences",
        ]
        for (tag, style), total_pages in analyzer.styles_unique_pages.most_common():
            styles_table.add_row(
                [tag, style, total_pages, analyzer.styles_total[(tag, style)]]
            )

        self.stdout.write("")
        self.stdout.write("Most commonly used inline CSS styles")
        self.stdout.write(str(styles_table))

        shortcodes_table = PrettyTable()
        shortcodes_table.field_names = [
            "Shortcode",
            "Pages used on",
            "Total occurrences",
        ]
        for shortcode, total_pages in analyzer.shortcodes_unique_pages.most_common():
            shortcodes_table.add_row(
                [shortcode, total_pages, analyzer.shortcodes_total[shortcode]]
            )

        self.stdout.write("Most commonly used shortcodes")
        self.stdout.write(str(shortcodes_table))

    def get_xml_file(self, xml_file):
        if os.path.exists(xml_file):
            return xml_file

        self.stdout.write(
            self.style.ERROR(f"The xml file cannot be found at: {xml_file}")
        )
        exit()



from django.urls import include, path
from django.views.i18n import JavaScriptCatalog
from wagtail.core import hooks


@hooks.register("register_admin_urls")
def register_admin_urls():
    urls = [
        path(
            "jsi18n/",
            JavaScriptCatalog.as_view(packages=["wagtail_wordpress_import"]),
            name="javascript_catalog",
        ),
        # Add your other URLs here, and they will appear under `/admin/xmlimport/`
        # Note: you do not need to check for authentication in views added here, Wagtail does this for you!
    ]

    return [
        path(
            "xmlimport/",
            include(
                (urls, "wagtail_wordpress_import"),
                namespace="wagtail_wordpress_import",
            ),
        )
    ]



from django.db import models
from wagtail.admin.edit_handlers import FieldPanel, FieldRowPanel
from wagtail.core.models import Page


class WPImportedPageMixin(Page):
    wp_post_id = models.IntegerField(blank=True, null=True)
    wp_post_type = models.CharField(max_length=255, blank=True, null=True)
    wp_link = models.TextField(blank=True, null=True)
    wp_raw_content = models.TextField(blank=True, null=True)
    wp_processed_content = models.TextField(blank=True, null=True)
    wp_block_json = models.TextField(blank=True, null=True)
    wp_normalized_styles = models.TextField(blank=True, null=True)
    wp_post_meta = models.JSONField(blank=True, null=True)

    class Meta:
        abstract = True

    def import_wordpress_data(self, data):
        raise NotImplementedError(
            "import_wordpress_data() method not implemented in your page model"
        )

    wordpress_panels = [
        FieldRowPanel(
            [
                FieldPanel("wp_post_id"),
                FieldPanel("wp_post_type"),
            ],
            heading="wp data",
        ),
        FieldPanel("wp_link", classname="full title"),
        FieldPanel("wp_block_json", classname="full"),
        FieldPanel("wp_processed_content", classname="full"),
        FieldPanel("wp_normalized_styles", classname="full"),
        FieldPanel("wp_raw_content", classname="full"),
        FieldPanel("wp_post_meta", classname="full"),
    ]



from django import forms
from django.db import models
from modelcluster.fields import ParentalManyToManyField
from wagtail.admin.edit_handlers import FieldPanel, StreamFieldPanel
from wagtail.core.fields import StreamField
from wagtail.core.models import Page
from wagtail.snippets.models import register_snippet
from wagtail_wordpress_import.blocks import WPImportStreamBlocks
from wagtail_wordpress_import.models import WPImportedPageMixin


class TestPage(WPImportedPageMixin, Page):
    body = StreamField(WPImportStreamBlocks)
    categories = ParentalManyToManyField("example.Category", blank=True)
    content_panels = Page.content_panels + [
        StreamFieldPanel("body"),
    ]

    def import_wordpress_data(self, data):
        # wagtail page model fields
        self.title = data["title"]
        self.slug = data["slug"]
        self.first_published_at = data["first_published_at"]
        self.last_published_at = data["last_published_at"]
        self.latest_revision_created_at = data["latest_revision_created_at"]
        self.search_description = data["search_description"]

        # debug fields
        self.wp_post_id = data["wp_post_id"]
        self.wp_post_type = data["wp_post_type"]
        self.wp_link = data["wp_link"]
        self.wp_raw_content = data["wp_raw_content"]
        self.wp_block_json = data["wp_block_json"]
        self.wp_processed_content = data["wp_processed_content"]
        self.wp_normalized_styles = data["wp_normalized_styles"]

        # own model fields
        self.body = data["body"]


@register_snippet
class Category(models.Model):
    name = models.CharField(max_length=255, unique=True)

    panels = [FieldPanel("name")]

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "category"
        verbose_name_plural = "categories"



# Generated by Django 3.2.7 on 2021-11-15 13:38

from django.db import migrations
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0003_add_unique_constraint_name'),
    ]

    operations = [
        migrations.AlterField(
            model_name='testpage',
            name='body',
            field=wagtail.core.fields.StreamField([('rich_text', wagtail.core.blocks.RichTextBlock(features=['anchor-identifier', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'bold', 'italic', 'ol', 'ul', 'hr', 'link', 'document-link', 'image', 'embed', 'superscript', 'subscript', 'strikethrough', 'blockquote'])), ('heading', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title')), ('importance', wagtail.core.blocks.ChoiceBlock(choices=[('h1', 'H1'), ('h2', 'H2'), ('h3', 'H3'), ('h4', 'H4'), ('h5', 'H5'), ('h6', 'H6')]))])), ('image', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock()), ('caption', wagtail.core.blocks.CharBlock(required=False)), ('link', wagtail.core.blocks.URLBlock(required=False)), ('alignment', wagtail.core.blocks.ChoiceBlock(choices=[('left', 'Left'), ('right', 'Right'), ('center', 'Center')]))])), ('block_quote', wagtail.core.blocks.StructBlock([('quote', wagtail.core.blocks.CharBlock(form_classname='title')), ('attribution', wagtail.core.blocks.CharBlock(required=False))])), ('raw_html', wagtail.core.blocks.RawHTMLBlock())]),
        ),
    ]



# Generated by Django 3.2.8 on 2021-10-18 10:59

from django.db import migrations, models
import django.db.models.deletion
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0062_comment_models_and_pagesubscription'),
    ]

    operations = [
        migrations.CreateModel(
            name='TestPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.page')),
                ('wp_post_id', models.IntegerField(blank=True, null=True)),
                ('wp_post_type', models.CharField(blank=True, max_length=255, null=True)),
                ('wp_link', models.TextField(blank=True, null=True)),
                ('wp_raw_content', models.TextField(blank=True, null=True)),
                ('wp_processed_content', models.TextField(blank=True, null=True)),
                ('wp_block_json', models.TextField(blank=True, null=True)),
                ('wp_normalized_styles', models.TextField(blank=True, null=True)),
                ('body', wagtail.core.fields.StreamField([('rich_text', wagtail.core.blocks.RichTextBlock(features=['anchor-identifier', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'bold', 'italic', 'ol', 'ul', 'hr', 'link', 'document-link', 'image', 'embed', 'superscript', 'subscript', 'strikethrough', 'blockquote'])), ('heading', wagtail.core.blocks.StructBlock([('text', wagtail.core.blocks.CharBlock(form_classname='title')), ('importance', wagtail.core.blocks.ChoiceBlock(choices=[('h1', 'H1'), ('h2', 'H2'), ('h3', 'H3'), ('h4', 'H4'), ('h5', 'H5'), ('h6', 'H6')]))])), ('image', wagtail.core.blocks.StructBlock([('image', wagtail.images.blocks.ImageChooserBlock()), ('caption', wagtail.core.blocks.CharBlock(required=False))])), ('block_quote', wagtail.core.blocks.StructBlock([('quote', wagtail.core.blocks.CharBlock(form_classname='title')), ('attribution', wagtail.core.blocks.CharBlock(required=False))])), ('raw_html', wagtail.core.blocks.RawHTMLBlock())])),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



import io
import tempfile
from PIL import Image
from bs4 import BeautifulSoup


def get_soup(html, parser):
    soup = BeautifulSoup(html, parser)
    return soup


def mock_image(file_name="test.jpg", width=100, height=100, color="white"):
    file = io.BytesIO()
    image = Image.new("RGB", (width, height), color)
    image.save(file, "JPEG")
    file.name = file_name
    file.seek(0)
    return file


def mock_pdf():
    temp_file = tempfile.NamedTemporaryFile(suffix=".pdf")
    temp_file.write(b"PDF Document")
    return open(temp_file.name, mode="rb")



import copy

from django.conf import settings


class ItemsCache:
    """Store the WordPress XML item tags.
    These are item tags that don't represent a page in the XML file.
    They are temporarily stored and used at the end of the import process.
    """

    def __init__(self):
        """For each value in settings.WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE
        create the class attribute.
        """

        for hook in getattr(
            settings, "WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE", {}
        ).keys():
            setattr(self, hook, [])

    def add_item_to_cache(self, key, item):
        """Add an item dict to the list of cached items if not already added.

        key:
            The value from <wp:post_type>...</wp:post_type> of an item tag in the XML file.
        item:
            The complete item tag in the XML file as a dict.
        """

        cache = getattr(self, key)
        item = copy.deepcopy(item)
        if "wp:postmeta" in item:  # wp:postmeta is not needed in the cache
            del item["wp:postmeta"]
        if item not in cache:
            cache.append(item)


class TagsCache:
    """Store the WordPress XML top level tags.
    These are tags that don't represent a page in the XML file.
    They are temporarily stored and used at the end of the import process.
    """

    def __init__(self):
        """For each value in settings.WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE
        create the class attribute.
        """

        for hook in getattr(settings, "WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE", {}):
            setattr(self, hook, [])

    def add_item_to_cache(self, key, item):
        """Add an item dict to the list of cached items if not already added

        key:
            The XML tag name of a tag in the XML file.
        item:
            The complete tag in the XML file as a dict.
        """

        cache = getattr(self, key)
        item = copy.deepcopy(item)
        if "wp:postmeta" in item:  # wp:postmeta is not needed in the cache
            del item["wp:postmeta"]
        if item not in cache:
            cache.append(item)



import json
import os

from django.core.management.base import BaseCommand
import collections
import sys
import re

from lxml import etree


class Command(BaseCommand):

    help = """Utils to output xml structure to a json file."""

    # def __init__(self, *args, **kwargs):
    #     # self.xml_folder_path = "xml"
    #     # self.json_folder_path = "json"
    #     if not os.path.exists(self.json_folder_path):
    #         os.makedirs(self.json_folder_path)
    #     super(Command, self).__init__(*args, **kwargs)

    def add_arguments(self, parser):
        parser.add_argument("xmlfile", type=str)

    def handle(self, *args, **options):
        self.stdout.write(self.style.WARNING("Parsing ..."))
        self.stdout.write(
            self.style.NOTICE(
                "If this is a large file and has many depths it will take some time to complete"
            )
        )
        file_name = options["xmlfile"]
        xml = open(file_name, "rb").read()

        json_file_name = file_name.split(".")[0]
        json_file = open(f"{json_file_name}.json", "w+")

        paths_dict = PathsToDict(xml).get_dict()

        json_file.write(json.dumps(paths_dict, indent=2))

        self.stdout.write(
            self.style.SUCCESS(f"Finished Your file is here: {json_file_name}.json")
        )


class PathsToDict:
    """
    Flatten the xml file tree.

    ...

    This is used when running the discovery command. The output has no original
    data and keys are xml tags and values are xml tag attribute names,
    genrated form the json file

    Attributes
    ----------
    xml_file: str
        the raw string xml file

    Methods
    -------
    get_dict()
        Traverse the xml tree.
        Check if the max_depth has been reached and if so stop and return
        a dict that represents all the unique tags of the xml file including
        attribute names

        Return collections.OrderedDict()

    """

    def __init__(self, xml_file):
        """
        Parameters
        ----------

        current_depth : int
            to keep track of the current depth during parsing
        max_depth : int
            the number of the deepest xml tags (tree)
        xml_root : lxml.etree
            the xml parsed and which we loop through
        raw_tree : str
            used when converting a tag to str representaion

        """
        self.current_depth = 0
        self.max_depth = MaxDepthEtree(xml_file)
        self.xml_root = etree.fromstring(xml_file)
        self.raw_tree = etree.ElementTree(self.xml_root)

    @staticmethod
    def get_path(tag, raw_tree):
        """
        Return a string: output a string like, /catalog/book/description
        from the tag path
        """

        return re.sub(r"\[[0-9]+\]", "", raw_tree.getpath(tag))

    @staticmethod
    def set_current_depth(path):
        """
        Count the current parts of path
        e.g. /catalog/book/description = 3
        """

        return len(str(path).strip("/").split("/"))

    @staticmethod
    def set_attrs(tag):
        """
        Extract any attribute names of a path
        """

        return ",".join([attr for attr in tag.keys()])

    def get_dict(self):
        """
        Traverse the xml tree
        """

        nice_tree = collections.OrderedDict()
        counter = 0

        for tag in self.xml_root.iter():
            counter += 1
            # out = f"({counter}) ... {tag}\n"
            # sys.stdout.write(out)
            path = self.get_path(tag, self.raw_tree)
            current_depth = self.set_current_depth(path)
            max_depth_reached = current_depth == self.max_depth

            if path in nice_tree and max_depth_reached:
                break

            if path not in nice_tree:
                nice_tree[path] = []

            if len(tag.keys()) > 0:
                nice_tree[path] = self.set_attrs(tag)

        return nice_tree


class MaxDepthEtree:
    """
    Calculate the maximum depth of an xml file tree.

    ...

    Traversing the tree until the root element is reached

    Attributes
    ----------
    xml : str
        the raw xml file

    Methods
    -------
    get_depth()
        Returns the calculated max depth of the xml file

    """

    def __init__(self, xml):
        """
        Parameters
        -----------

        max_depth : int
            keep track of the deepest so far
        tree : the xml tree to parse

        """
        self.max_depth = 0
        self.tree = etree.ElementTree(etree.fromstring(xml))

    def _depth(self, elem, level):
        """
        Recursive method to count the current depth
        """

        if level == self.max_depth:
            self.max_depth += 1
        for child in elem:
            self._depth(child, level + 1)

    def get_depth(self):
        """
        # returning 1 greater than actual length to work with
        """
        self._depth(self.tree.getroot(), -1)
        return self.max_depth + 1



from django.test import TestCase

from wagtail_wordpress_import.handle_inline_shortcodes import InlineShortcodeHandler


class TestInlineShortcodeHandler(TestCase):
    def test_regex_has_matches(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

            def construct_html_tag(self, html):
                return self._pattern.finditer(html)

        handler = FooHandler()
        html = """[foo bar="1"]
        [bar foo="1"]"""

        out = handler.construct_html_tag(html)
        self.assertEqual(sum(1 for _ in out), 1)

    def test_regex_has_no_matches(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "nomatch"

            def construct_html_tag(self, html):
                return self._pattern.finditer(html)

        handler = FooHandler()
        html = """[foo bar="1"]
        [bar foo="1"]"""

        out = handler.construct_html_tag(html)
        self.assertEqual(sum(1 for _ in out), 0)

    def test_regex_mathes_adjacent_first_place(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

            def construct_html_tag(self, html):
                return self._pattern.finditer(html)

        handler = FooHandler()
        html = """[foo bar="1"][bar foo="1"]"""

        out = handler.construct_html_tag(html)
        self.assertEqual(sum(1 for _ in out), 1)

    def test_regex_mathes_adjacent_content_between(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "bar"

            def construct_html_tag(self, html):
                return self._pattern.finditer(html)

        handler = FooHandler()
        html = """[foo bar="1"]content between[bar foo="1"]"""

        out = handler.construct_html_tag(html)
        self.assertEqual(sum(1 for _ in out), 1)

    def test_regex_mathes_adjacent_content_around(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "bar"

            def construct_html_tag(self, html):
                return self._pattern.finditer(html)

        handler = FooHandler()
        html = """content before[foo bar="1"][bar foo="1"]content after"""

        out = handler.construct_html_tag(html)
        self.assertEqual(sum(1 for _ in out), 1)


class TestInlineShortcodeHandlerDocumentedExample(TestCase):
    def test_construct_html_tag_documented_example(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

            def construct_html_tag(self, html):
                matches = self._pattern.finditer(html)
                for match in matches:
                    attrs = self.get_shortcode_attrs(match.groupdict()["attrs"])
                    html = html.replace(
                        match.group(),
                        f'<{self.element_name} data-{self.shortcode_name}="{attrs["symbol"]}">${attrs["symbol"]}</{self.element_name}>',
                    )

                return html

        handler = FooHandler()
        html = """some content before [foo symbol="ABC"] some content after"""

        out = handler.construct_html_tag(html)
        self.assertEqual(
            out,
            'some content before <span data-foo="ABC">$ABC</span> some content after',
        )

    def test_construct_html_tag_documented_example_ignores_bar(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

            def construct_html_tag(self, html):
                matches = self._pattern.finditer(html)
                for match in matches:
                    attrs = self.get_shortcode_attrs(match.groupdict()["attrs"])
                    html = html.replace(
                        match.group(),
                        f'<{self.element_name} data-{self.shortcode_name}="{attrs["symbol"]}">${attrs["symbol"]}</{self.element_name}>',
                    )

                return html

        handler = FooHandler()
        html = """some content before [foo symbol="ABC"] some content after [bar baz="foo"]"""

        out = handler.construct_html_tag(html)
        self.assertEqual(
            out,
            'some content before <span data-foo="ABC">$ABC</span> some content after [bar baz="foo"]',
        )


class TestGetAttrs(TestCase):
    def test_get_shortcode_attrs(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

        handler = FooHandler()
        attrs = 'foo="1" bar="2"'

        attrs = handler.get_shortcode_attrs(attrs)
        self.assertEqual(attrs, {"foo": "1", "bar": "2"})

    def test_get_shortcode_attrs_missing_quotes(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

        handler = FooHandler()
        attrs = 'foo="1" bar=2'

        attrs = handler.get_shortcode_attrs(attrs)
        self.assertEqual(attrs, {"foo": "1", "bar": "2"})

    def test_get_shortcode_attrs_no_quotes(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

        handler = FooHandler()
        attrs = "foo=1 bar=2"

        attrs = handler.get_shortcode_attrs(attrs)
        self.assertEqual(attrs, {"foo": "1", "bar": "2"})

    def test_get_shortcode_attrs_extra_spacing(self):
        class FooHandler(InlineShortcodeHandler):
            shortcode_name = "foo"

        handler = FooHandler()
        attrs = " foo=1  bar=2 "

        with self.subTest(
            """The provided method cannot handle multiple spaces around
            the attrs or multiple space between the attrs"""
        ):
            with self.assertRaises(IndexError):
                attrs = handler.get_shortcode_attrs(attrs)



import re

from django.utils.html import escape
from django.utils.text import normalize_newlines


def filter_linebreaks_wp(pee, options=None):
    """
    Straight up port of http://codex.wordpress.org/Function_Reference/wpautop
    i am greatful https://gist.github.com/albertsun/1160201

    param: `options` NOT IMPLEMENTED
    """
    # print(pee)
    # seeing an error here when the first charcter is a int
    # if (pee.strip() == ""):
    #     return ""
    pee = normalize_newlines(pee)
    pee = pee + "\n"
    pee = re.sub(r"<br />\s*<br />", "\n\n", pee)
    allblocks = r"(?:table|thead|tfoot|caption|col|colgroup|tbody|tr|td|th|div|dl|dd|dt|ul|ol|li|pre|select|option|form|map|area|blockquote|address|math|style|input|p|h[1-6]|hr|fieldset|legend|section|article|aside|hgroup|header|footer|nav|figure|figcaption|details|menu|summary)"
    pee = re.sub(
        r"(<" + allblocks + "[^>]*>)",
        lambda m: "\n" + m.group(1) if m.group(1) else "\n",
        pee,
    )
    pee = re.sub(
        r"(</" + allblocks + ">)",
        lambda m: m.group(1) + "\n\n" if m.group(1) else "\n\n",
        pee,
    )
    # pee = pee.replace("\r\n", "\n")
    # pee = pee.replace("\r", "\n") #these taken care of by normalize_newlines
    if pee.find("<object") != -1:
        pee = re.sub(
            r"\s*<param([^>]*)>\s*",
            lambda m: "<param%s>" % (m.group(1) if m.group(1) else "",),
            pee,
        )  # no pee inside object/embed
        pee = re.sub(r"\s*</embed>\s*", "</embed>", pee)
    pee = re.sub(r"\n\n+", "\n\n", pee)  # take care of duplicates
    pees = re.split(
        r"\n\s*\n", pee
    )  # since PHP has a PREG_SPLIT_NO_EMPTY, may need to go through and drop any empty strings
    # pees = [p for p in pees if p]
    pee = "".join(["<p>%s</p>\n" % tinkle.strip("\n") for tinkle in pees])
    pee = re.sub(
        r"<p>\s*</p>", "", pee
    )  # under certain strange conditions it could create a P of entirely whitespace
    pee = re.sub(
        r"<p>([^<]+)</(div|address|form)>",
        lambda m: "<p>%s</p></%s>"
        % (
            (lambda x: x.group(1) if x.group(1) else "")(m),
            (lambda x: x.group(2) if x.group(2) else "")(m),
        ),
        pee,
    )
    pee = re.sub(
        r"<p>\s*(</?" + allblocks + r"[^>]*>)\s*</p>",
        lambda m: m.group(1) if m.group(1) else "",
        pee,
    )  # don't pee all over a tag
    pee = re.sub(
        r"<p>(<li.+?)</p>", lambda m: m.group(1) if m.group(1) else "", pee
    )  # problem with nested lists
    pee = re.sub(
        r"<p><blockquote([^>]*)>",
        # lambda m: "<blockquote%s><p>" % (m.group(1) if m.group(1) else "",),
        # found this not to our liking as it puts a p tag around the blockquote content
        lambda m: "<blockquote%s>" % (m.group(1) if m.group(1) else "",),
        pee,
        flags=re.IGNORECASE,
    )
    # pee = pee.replace("</blockquote></p>", "</p></blockquote>")
    # found this not to our liking as it puts a p tag around the blockquote content
    pee = pee.replace("</blockquote></p>", "</blockquote>")
    pee = re.sub(
        r"<p>\s*(</?" + allblocks + r"[^>]*>)",
        lambda m: m.group(1) if m.group(1) else "",
        pee,
    )
    pee = re.sub(
        r"(</?" + allblocks + "[^>]*>)\s*</p>",
        lambda m: m.group(1) if m.group(1) else "",
        pee,
    )

    def _autop_newline_preservation_helper(matches):
        return matches.group(0).replace("\n", "<WPPreserveNewline />")

    pee = re.sub(
        r"<(script|style).*?</\1>",
        _autop_newline_preservation_helper,
        pee,
        flags=re.DOTALL,
    )
    pee = re.sub(r"(?<!<br />)\s*\n", "<br />\n", pee)  # make line breaks
    pee = pee.replace("<WPPreserveNewline />", "\n")

    pee = re.sub(
        r"(</?" + allblocks + "[^>]*>)\s*<br />",
        lambda m: m.group(1) if m.group(1) else "",
        pee,
    )
    pee = re.sub(
        r"<br />(\s*</?(?:p|li|div|dl|dd|dt|th|pre|td|ul|ol)[^>]*>)",
        lambda m: m.group(1) if m.group(1) else "",
        pee,
    )
    if pee.find("<pre") != -1:

        def clean_pre(m):
            if m.group(1) and m.group(2):
                text = m.group(2)
                text = text.replace("<br />", "")
                text = text.replace("<p>", "\n")
                text = text.replace("</p>", "")
                text = m.group(1) + escape(text) + "</pre>"
            else:
                text = m.group(0)
                text = text.replace("<br />", "")
                text = text.replace("<p>", "\n")
                text = text.replace("</p>", "")

            return text

        pee = re.sub("(?is)(<pre[^>]*>)(.*?)</pre>", clean_pre, pee)
    pee = re.sub(r"\n</p>$", "</p>", pee)
    return pee



# Generated by Django 3.2.7 on 2021-10-25 11:53

from django.db import migrations, models
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
            ],
            options={
                'verbose_name': 'category',
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.AddField(
            model_name='testpage',
            name='categories',
            field=modelcluster.fields.ParentalManyToManyField(blank=True, to='example.Category'),
        ),
    ]



from collections import defaultdict


def clean_node_name(node_name):
    return node_name.replace("-", "_")


def coerce_node_value(value):
    if value.isnumeric():
        return int(value)
    if value == "true":
        return True
    if value == "false":
        return False
    return value


def get_node_value(node):
    if len(node.childNodes) == 0:
        return node.nodeName, None
    else:
        contains_element_node = False
        for child_node in node.childNodes:
            if child_node.nodeType == node.ELEMENT_NODE:
                contains_element_node = True
                break
        if contains_element_node:
            return node.nodeName, node_to_dict(node)
        else:
            return node.nodeName, coerce_node_value(
                "".join(child_node.nodeValue for child_node in node.childNodes)
            )


def node_to_dict(node):
    obj = defaultdict(list)
    for child_node in node.childNodes:
        if child_node.nodeType == node.ELEMENT_NODE:
            name, value = get_node_value(child_node)
            obj[clean_node_name(name)].append(value)
        elif child_node.nodeType == node.TEXT_NODE:
            pass  # Usually whitespace
        else:
            raise Exception()
    # If an element appears more than once, use an array.
    # Otherwise just use the element value.
    obj = {key: value[0] if len(value) == 1 else value for key, value in obj.items()}
    if obj == {"nil": True}:
        return None
    return obj


def snakecase_key(key):
    """
    Convert the key to snake_case by replacing ':' with '_'
    and remove any leading '_'
    This is so the keys are valid as Python kwargs and can be used to filter
    the page models on on keys in the wp_post_meta which is a JSONField().
    """
    return (
        str(key)  # some keys look like boolean types
        .replace(":", "_")  # some keys contain ':'
        .lstrip("_")  # some keys start with '_'
    )


def get_attr_as_list(node, key):
    try:
        if key in node:
            if len(node[key]) == 0:
                return []
            if isinstance(node[key], dict):
                return [node[key]]
            return node[key]
        return []
    except TypeError:
        return []



import re

from wagtail_wordpress_import.block_builder_defaults import get_or_save_image

SHORTCODE_HANDLERS = []


def register():
    """Register the decorated class as a shortcode handler.

    Usage:

        from wagtail_wordpress_import.prefilters.handle_shortcodes import BlockShortcodeHandler, register

        @register("foo")
        class MyShortcodeHandler(BlockShortcodeHandler):
            shortcode_name = "foo"
    """

    def _wrapper(cls):
        SHORTCODE_HANDLERS.append(cls)
        return cls

    return _wrapper


class BlockShortcodeHandler:

    shortcode_name: str

    is_top_level_html_tag = True

    def __init__(self):
        # Subclasses should declare a shortcode_name
        if not hasattr(self, "shortcode_name"):
            raise NotImplementedError(
                "Create a subclass of BlockShortcodeHandler with a shortcode_name attribute"
            )
        pattern = re.compile(r"^\S[a-zA-Z0-9_\S]+\S$")
        # shortcode_name must use upper or lower case letters or digits and cannot contain spaces
        if not re.match(pattern, self.shortcode_name):
            raise ValueError(
                "The shortcode_name attribute must use upper or lower case letters or digits and cannot contain spaces"
            )

    @property
    def _pattern(self):
        """Return a regex to match a block shortcode and capture the attrs and text.

        Given an input:

            "Preface [foo bar=1]some text[/foo] epilogue."

        the regex will match the string between the two "foo" tags, inclusively. The
        capture group "attrs" will match " bar=1", and capture group "content" will
        match "some text".
        """

        return re.compile(
            r"\["  # matches the opening [
            + self.shortcode_name
            + r"\b"  # matches a word boundary
            + r"(?P<attrs>[^\]]*)"  # capture 'attrs', matching anything but ]
            + r"\]"  # matches the closing ] of the opening tag
            + r"(?P<content>.*?)"  # non-greedily captures 'content' between the tags
            + r"\[\/"  # matches the  [/ of the closing tag
            + self.shortcode_name
            + r"\]"  # matches the closing ]
        )

    def pre_filter(self, string):
        """Replace all occurrences of the tag with a HTML tag for later parsing.

        Given an input:

            "Preface [foo bar=1]some text[/foo] epilogue."

        this function will return

            "Preface <wagtail_block_foo bar=1>some text</wagtail_block_foo> epilogue."
        """
        string, matches = self._pattern.subn(
            r"<"
            + self.element_name
            + r"\g<attrs>>\g<content></"
            + self.element_name
            + r">",
            string,
        )

        return string

    @property
    def element_name(self):
        return f"wagtail_block_{self.shortcode_name}"


# Subclasses should declare a shortcode_name and provide
# a construct_block method for converting the pre-filtered HTML to a
# Wagtail StreamField block dict.
@register()
class CaptionHandler(BlockShortcodeHandler):
    """
    The Wordpress caption tag is replaced by the custom HTML tag. The caption content and caption attrubutes
    are preserved and included in the custom HTML tag.

    Sample wordpress caption tag:

    [caption id="attachment_46162" align="aligncenter" width="600"]
    <a href="http://www.example.com/">
    <img class="wp-image-46162 size-full" src="https://www.example.com/images/foo.jpg" alt="This describes the image" width="600" height="338" />
    </a>
    This is a caption about the image[/caption]

    is replaced by:

    <wagtail_block_caption id="attachment_46162" align="aligncenter" width="600">
    <a href="http://www.example.com/">
    <img class="wp-image-46162 size-full" src="https://www.example.com/images/foo.jpg" alt="This describes the image" width="600" height="338" />
    </a>
    This is a caption about the image</wagtail_block_caption>

    in the pre-filter method on the parent BlockShortcodeHandler class.
    """

    shortcode_name = "caption"

    def construct_block(self, soup):
        """Construct a StreamBlock dict that's passed back to the block builder

        soup: <class 'bs4.element.Tag'>
        """

        # Without an image the caption shortcode is useless
        # Here we'll add a raw HTML block to the stream so in admin
        # it can be seen and some action taken.
        image = soup.find("img")
        if not image:
            return {
                "type": "raw_html",
                "value": f"<!-- No image found in caption {soup}-->",
            }
        # TODO: Output this in logging when we have a logger
        # https://projects.torchbox.com/projects/wordpress-to-wagtail-importer-package/tickets/63

        # Parse the image
        image_file = get_or_save_image(image.attrs["src"])

        # Parse alignment
        if "align" in soup.attrs:
            alignment = soup.attrs["align"].replace("align", "")
        else:
            alignment = "left"

        # Parse the caption
        caption = soup.text.replace("\n", "").strip()

        # Parse the anchor link
        # The template has a conditional that only render the link
        # if the return value["link"] has a value
        anchor = soup.find("a")
        if anchor:
            link = anchor.attrs["href"]
        else:
            link = ""

        return {
            "type": "image",
            "value": {
                "image": image_file.id,
                "caption": caption,
                "alignment": alignment,
                "link": link,
            },
        }


def filter_transform_shortcodes(html, options=None):
    """
    html: is the body content from one Wordpress item
    options: not implemented
    """
    for handler in SHORTCODE_HANDLERS:
        html = handler().pre_filter(html)
    return html



# Generated by Django 3.2.7 on 2021-11-02 17:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0002_add_categories'),
    ]

    operations = [
        migrations.AlterField(
            model_name='category',
            name='name',
            field=models.CharField(max_length=255, unique=True),
        ),
    ]



import os

from django.test import TestCase
from wagtail_wordpress_import.prefilters.bleach_filter import filter_bleach_clean

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"


class TestBleach(TestCase):
    def setUp(self):
        self.raw_html_file = open(f"{FIXTURES_PATH}/raw_html.txt", "r")
        self.stream = self.raw_html_file.read()

    def test_bleach_clean(self):
        bc = filter_bleach_clean(self.stream)
        self.assertNotIn('style=" float: left;"', bc)
        self.assertNotIn('onmouseover=alert("Boo!")', bc)



from collections import Counter

from django.test.html import HTMLParseError, parse_html

from .shortcodes import find_all_shortcodes


class HTMLAnalyzer:
    def __init__(self):
        self.total = 0
        self.invalid = 0
        self.tags_total = Counter()
        self.attributes_total = Counter()
        self.styles_total = Counter()
        self.classes_total = Counter()
        self.shortcodes_total = Counter()

        self.tags_unique_pages = Counter()
        self.attributes_unique_pages = Counter()
        self.styles_unique_pages = Counter()
        self.classes_unique_pages = Counter()
        self.shortcodes_unique_pages = Counter()

    @classmethod
    def find_all_tags(cls, dom):
        names = Counter()
        for child in dom.children:
            if isinstance(child, str):
                continue

            names[child.name[:30]] += 1
            names.update(cls.find_all_tags(child))

        return names

    @classmethod
    def find_all_attributes(cls, dom):
        attrs = Counter()
        for child in dom.children:
            if isinstance(child, str):
                continue

            for attr_name, attr_value in child.attributes:
                attrs[(child.name[:30], attr_name[:30])] += 1

            attrs.update(cls.find_all_attributes(child))

        return attrs

    @classmethod
    def find_all_styles(cls, dom):
        styles = Counter()
        for child in dom.children:
            if isinstance(child, str):
                continue

            for attr_name, attr_value in child.attributes:
                if attr_name == "style":
                    for rule in attr_value.split(";"):
                        if rule.strip():
                            styles[(child.name[:30], rule.strip()[:30])] += 1

            styles.update(cls.find_all_styles(child))

        return styles

    @classmethod
    def find_all_classes(cls, dom):
        classes = Counter()
        for child in dom.children:
            if isinstance(child, str):
                continue

            for attr_name, attr_value in child.attributes:
                if attr_name == "class":
                    for class_name in attr_value.split(" "):
                        classes[(child.name[:30], class_name.strip()[:30])] += 1

            classes.update(cls.find_all_classes(child))

        return classes

    @classmethod
    def find_all_shortcodes(cls, dom):
        shortcodes = Counter()
        for child in dom.children:
            if isinstance(child, str):
                shortcodes.update(find_all_shortcodes(child))
            else:
                shortcodes.update(cls.find_all_shortcodes(child))

        return shortcodes

    def analyze(self, html):
        self.total += 1

        try:
            dom = parse_html(html)
        except HTMLParseError:
            self.invalid += 1
            return

        tags = self.find_all_tags(dom)
        attributes = self.find_all_attributes(dom)
        styles = self.find_all_styles(dom)
        classes = self.find_all_classes(dom)
        shortcodes = self.find_all_shortcodes(dom)

        self.tags_total.update(tags)
        self.attributes_total.update(attributes)
        self.styles_total.update(styles)
        self.classes_total.update(classes)
        self.shortcodes_total.update(shortcodes)

        self.tags_unique_pages.update(tags.keys())
        self.attributes_unique_pages.update(attributes.keys())
        self.styles_unique_pages.update(styles.keys())
        self.classes_unique_pages.update(classes.keys())
        self.shortcodes_unique_pages.update(shortcodes.keys())



from django.conf import settings


def debug_enabled():
    return getattr(settings, "WAGTAIL_WORDPRESS_IMPORT_DEBUG_ENABLED", True)


def yoast_plugin_enabled():
    return getattr(settings, "WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_ENABLED", False)


def yoast_plugin_config():
    """
    XML file fields
    <wp:postmeta>
        <wp:meta_key>_yoast_wpseo_metadesc</wp:meta_key>
        <wp:meta_value>a search description from yaost for Item two</wp:meta_value>
    </wp:postmeta>
    """
    return getattr(
        settings,
        "WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_MAPPING",
        {
            "xml_item_key": "wp:postmeta",
            "description_key": "wp:meta_key",
            "description_value": "wp:meta_value",
            "description_key_value": "_yoast_wpseo_metadesc",
        },
    )


def category_plugin_enabled():
    return getattr(settings, "WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_ENABLED", False)


def category_name_min_length():
    return getattr(
        settings, "WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_MIN_NAME_LENGTH", 1
    )


def get_category_model():
    return getattr(settings, "WAGTAIL_WORDPRESS_IMPORT_CATEGORY_PLUGIN_MODEL", None)



import csv
import sys
from datetime import datetime


class Logger:
    def __init__(self, logdir):
        self.logdir = logdir
        self.processed = 0
        self.imported = 0
        self.skipped = 0
        self.items = []
        self.images = []
        self.urls = []
        self.page_link_errors = []

    def log_progress(self):
        item = self.items[-1]
        if not item["id"] == 0:
            sys.stdout.write(
                f"Wagtail ID: {item['id']}, {item['title']}, {item['result']}\n"
            )

    def get_items_report_data(self):
        report_data = {
            "processed": self.processed,
            "imported": self.imported,
            "skipped": self.skipped,
            "items": self.items,
        }

        return report_data

    def output_import_summary(self):
        sys.stdout.write("Summary ========================")
        sys.stdout.write(
            "\nImported: "
            + str(self.imported)
            + " Skipped: "
            + str(self.skipped)
            + " Processed: "
            + str(self.processed)
        )
        if self.processed - self.skipped == self.imported:
            sys.stdout.write("\n✅ Completed Successfully\n")
        else:
            sys.stdout.write(
                "\n⚠️ Completed but there were errors with imported amounts\n"
            )

    def save_csv_import_report(self):
        file_name = f"{self.logdir}/import-report-{datetime.now().strftime('%Y%m%d-%H%M%S')}.csv"

        with open(file_name, "w", newline="") as csvfile:
            writer = csv.DictWriter(
                csvfile,
                fieldnames=[
                    "id",
                    "title",
                    "url",
                    "reason",
                    "result",
                    "dates",
                    "slug",
                ],
            )
            writer.writerow(
                {
                    "id": "Page ID",
                    "title": "Page Title",
                    "url": "Wordpress Link",
                    "reason": "Reason for result ->",
                    "result": "Result",
                    "dates": "Dates Changed",
                    "slug": "Slug Changed",
                }
            )
            for row in self.items:
                writer.writerow(
                    {
                        "id": row["id"],
                        "title": row["title"],
                        "url": row["link"],
                        "reason": row["reason"],
                        "result": row["result"],
                        "dates": row["datecheck"],
                        "slug": row["slugcheck"],
                    }
                )

    def save_csv_images_report(self):
        file_name = f"{self.logdir}/images-report-{datetime.now().strftime('%Y%m%d-%H%M%S')}.csv"

        with open(file_name, "w", newline="") as csvfile:
            writer = csv.DictWriter(
                csvfile,
                fieldnames=[
                    "id",
                    "title",
                    "url",
                    "reason",
                ],
            )
            writer.writerow(
                {
                    "id": "Page ID",
                    "title": "Page Title",
                    "url": "Wordpress Link",
                    "reason": "Reason",
                }
            )
            for row in self.images:
                writer.writerow(
                    {
                        "id": row["id"],
                        "title": row["title"],
                        "url": row["link"],
                        "reason": row["reason"],
                    }
                )

    def save_csv_pagelink_errors_report(self):
        file_name = f"{self.logdir}/pagelink_errors-report-{datetime.now().strftime('%Y%m%d-%H%M%S')}.csv"

        with open(file_name, "w", newline="") as csvfile:
            writer = csv.DictWriter(
                csvfile,
                fieldnames=[
                    "id",
                    "title",
                    "link",
                ],
            )
            writer.writerow(
                {
                    "id": "Page ID",
                    "title": "Page Title",
                    "link": "Wordpress Link",
                }
            )
            for row in self.images:
                writer.writerow(
                    {
                        "id": row["id"],
                        "title": row["title"],
                        "link": row["link"],
                    }
                )



import os
from io import StringIO
from unittest import mock
from xml.dom import pulldom

from django.conf import settings
from django.test import TestCase, override_settings
from wagtail.core.models import Page
from wagtail_wordpress_import.functions import node_to_dict
from wagtail_wordpress_import.importers.wordpress import WordpressImporter
from wagtail_wordpress_import.logger import Logger
from wagtail_wordpress_import.test.tests.xml_boilerplate import (
    build_xml_stream,
    generate_temporay_file,
)

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"
LOG_DIR = "fakedir"


class TestImportHooksOverrideItemTagConfig(TestCase):
    """The overridden conf should be return by import_hooks_xml_items_to_cache"""

    @override_settings(
        WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE={
            "foo": {"DATA_TAG": "datatagname", "FUNCTION": "path.to.function"},
            "bar": {"DATA_TAG": "datatagname", "FUNCTION": "path.to.function"},
        }
    )
    def test_persist_tags_config(self):
        self.assertEqual(
            getattr(settings, "WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE", {}),
            {
                "foo": {"DATA_TAG": "datatagname", "FUNCTION": "path.to.function"},
                "bar": {"DATA_TAG": "datatagname", "FUNCTION": "path.to.function"},
            },
        )


@override_settings(
    WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE={
        "foo": {"DATA_TAG": "datatagname", "FUNCTION": "path.to.function"},
        "bar": {"DATA_TAG": "datatagname", "FUNCTION": "path.to.function"},
    },
)
class TestImportHooksXmlItemPersisted(TestCase):
    """The overridden config should result in ItemsCache instance contaiing two
    dicts with keys foo and bar. Each dict should contain all xml tags and
    values from the fragment"""

    def setUp(self):
        self.logger = Logger("foo")

    def process_item(self, xml_stream):
        """Turn an XML stream into a node dict
        This is duplicating a bunch of code from WordPressImporter. To test the ItemsCache
        instance on WordPressImporter.
        """
        self.importer = WordpressImporter(xml_stream)
        xml_doc = pulldom.parse(xml_stream)
        for event, node in xml_doc:
            if event == pulldom.START_ELEMENT and node.tagName == "item":
                xml_doc.expandNode(node)
                item = node_to_dict(node)
                post_type = item.get("wp:post_type")
                if post_type in getattr(
                    settings, "WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE", {}
                ):
                    self.importer.items_cache.add_item_to_cache(post_type, item)

        return self.importer.items_cache

    def test_xml_items_are_cached_without_duplicates(self):
        fragment = """
        <item>
            <title>foo-item</title>
            <link>https://www.example.com/foo-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/foo.jpg</guid>
            <wp:post_id>100</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>foo-item</wp:post_name>
            <wp:post_type>foo</wp:post_type>
        </item>
        <item>
            <title>foo-item</title>
            <link>https://www.example.com/foo-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/foo.jpg</guid>
            <wp:post_id>100</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>foo-item</wp:post_name>
            <wp:post_type>foo</wp:post_type>
        </item>
        <item>
            <title>bar-item</title>
            <link>https://www.example.com/bar-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/bar.jpg</guid>
            <wp:post_id>200</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>bar-item</wp:post_name>
            <wp:post_type>bar</wp:post_type>
        </item>
        """
        built = build_xml_stream(xml_items_fragment=fragment).read()
        cache = self.process_item(StringIO(built))

        self.assertIsInstance(cache.foo, list)

        # Duplicates should be skipped.
        # In the fragment we have identical items with title foo-item
        self.assertEqual(len(cache.foo), 1)

        # Values of the first item in the ItemsCache instance foo attribute
        foo = cache.foo[0]

        # there should be no wp:postmeta in the cache
        with self.assertRaises(KeyError):
            foo["wp:postmeta"]

        self.assertEqual(foo["title"], "foo-item")
        self.assertEqual(foo["link"], "https://www.example.com/foo-item/")
        self.assertEqual(foo["pubDate"], "Tue, 13 Jul 2010 16:16:46 +0000")
        self.assertEqual(foo["guid"], "https://www.example.com/foo.jpg")
        self.assertEqual(foo["wp:post_id"], 100)
        self.assertEqual(foo["wp:post_date"], "2010-07-13 12:16:46")
        self.assertEqual(foo["wp:post_date_gmt"], "2010-07-13 16:16:46")
        self.assertEqual(foo["wp:post_modified"], "2010-07-13 12:16:46")
        self.assertEqual(foo["wp:post_modified_gmt"], "2010-07-13 16:16:46")
        self.assertEqual(foo["wp:post_name"], "foo-item")
        self.assertEqual(foo["wp:post_type"], "foo")

        self.assertIsInstance(cache.bar, list)
        self.assertEqual(len(cache.bar), 1)

        # Values of the first item in the ItemsCache instance bar attribute
        bar = cache.bar[0]

        # there should be no wp:postmeta in the cache
        with self.assertRaises(KeyError):
            bar["wp:postmeta"]

        self.assertEqual(bar["title"], "bar-item")
        self.assertEqual(bar["link"], "https://www.example.com/bar-item/")
        self.assertEqual(bar["pubDate"], "Tue, 13 Jul 2010 16:16:46 +0000")
        self.assertEqual(bar["guid"], "https://www.example.com/bar.jpg")
        self.assertEqual(bar["wp:post_id"], 200)
        self.assertEqual(bar["wp:post_date"], "2010-07-13 12:16:46")
        self.assertEqual(bar["wp:post_date_gmt"], "2010-07-13 16:16:46")
        self.assertEqual(bar["wp:post_modified"], "2010-07-13 12:16:46")
        self.assertEqual(bar["wp:post_modified_gmt"], "2010-07-13 16:16:46")
        self.assertEqual(bar["wp:post_name"], "bar-item")
        self.assertEqual(bar["wp:post_type"], "bar")


class WordpressImporterTestsCheckXmlItemsNotCached(TestCase):
    """
    This tests that the XML tags are not cached during an import as there
    is no settings.WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE
    The item with <wp:post_type>post</wp:post_type> should be saved as a page
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def process_import(self, xml_stream):
        self.importer = WordpressImporter(xml_stream)
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages="example",
            model_for_pages="TestPage",
            parent_id=2,
            page_types=["post"],
            page_statuses=["publish"],
        )
        self.parent_page = Page.objects.get(id=2)
        self.imported_pages = self.parent_page.get_children().all()

    def test_xml_items_not_cached(self):
        fragment = """
        <item>
            <title>foo-item</title>
            <link>https://www.example.com/foo-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/foo.jpg</guid>
            <wp:post_id>100</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>foo-item</wp:post_name>
            <wp:post_type>foo</wp:post_type>
        </item>
        <item>
            <title>bar-item</title>
            <link>https://www.example.com/bar-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/bar.jpg</guid>
            <wp:post_id>200</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>bar-item</wp:post_name>
            <wp:post_type>bar</wp:post_type>
        </item>
        <item>
            <title>A title</title>
            <link>https://www.example.com/a-title</link>
            <description />
            <content:encoded />
            <excerpt:encoded />
            <wp:post_id>44221</wp:post_id>
            <wp:post_date>2015-05-21 15:00:31</wp:post_date>
            <wp:post_date_gmt>2015-05-21 19:00:31</wp:post_date_gmt>
            <wp:post_modified>2015-05-21 15:00:44</wp:post_modified>
            <wp:post_modified_gmt>2015-05-21 19:00:44</wp:post_modified_gmt>
            <wp:comment_status>open</wp:comment_status>
            <wp:ping_status>closed</wp:ping_status>
            <wp:post_name>a-title</wp:post_name>
            <wp:status>publish</wp:status>
            <wp:post_type>post</wp:post_type>
        </item>
        """
        built_file = generate_temporay_file(
            build_xml_stream(xml_items_fragment=fragment).read()
        )

        self.process_import(built_file)

        # One page should be imported
        self.assertEqual(self.imported_pages.count(), 1)
        self.assertEqual(self.imported_pages.first().title, "A title")

        # ItemsCache instance should have no writable attributes
        self.assertEqual(len(self.importer.items_cache.__dict__.keys()), 0)


process_foo = mock.Mock()
process_bar = mock.Mock()


@override_settings(
    WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE={
        "foo": {
            "DATA_TAG": "datatagname",
            "FUNCTION": "wagtail_wordpress_import.test.tests.test_import_hooks_items.process_foo",
        },
        "bar": {
            "DATA_TAG": "datatagname",
            "FUNCTION": "wagtail_wordpress_import.test.tests.test_import_hooks_items.process_bar",
        },
    },
)
class WordpressImporterTestsCheckXmlItemsCached(TestCase):
    """
    This tests that the XML tags are cached during an import because
    settings.WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE is set.
    The process run here is WordpressImporter.run()
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def process_import(self, xml_stream):
        self.importer = WordpressImporter(xml_stream)
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages="example",
            model_for_pages="TestPage",
            parent_id=2,
            page_types=["post"],
            page_statuses=["publish"],
        )

    def test_xml_items_cached(self):
        """There is a place in WordPressImporter thats runs add_item_to_cache()
        that will populate the ItemsCache instance attributes"""
        fragment = """
        <item>
            <title>foo-item</title>
            <link>https://www.example.com/foo-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/foo.jpg</guid>
            <wp:post_id>100</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>foo-item</wp:post_name>
            <wp:post_type>foo</wp:post_type>
        </item>
        <item>
            <title>bar-item</title>
            <link>https://www.example.com/bar-item/</link>
            <pubDate>Tue, 13 Jul 2010 16:16:46 +0000</pubDate>
            <guid isPermaLink="false">https://www.example.com/bar.jpg</guid>
            <wp:post_id>200</wp:post_id>
            <wp:post_date>2010-07-13 12:16:46</wp:post_date>
            <wp:post_date_gmt>2010-07-13 16:16:46</wp:post_date_gmt>
            <wp:post_modified>2010-07-13 12:16:46</wp:post_modified>
            <wp:post_modified_gmt>2010-07-13 16:16:46</wp:post_modified_gmt>
            <wp:post_name>bar-item</wp:post_name>
            <wp:post_type>bar</wp:post_type>
        </item>
        <item>
            <title>A title</title>
            <link>https://www.example.com/a-title</link>
            <description />
            <content:encoded />
            <excerpt:encoded />
            <wp:post_id>44221</wp:post_id>
            <wp:post_date>2015-05-21 15:00:31</wp:post_date>
            <wp:post_date_gmt>2015-05-21 19:00:31</wp:post_date_gmt>
            <wp:post_modified>2015-05-21 15:00:44</wp:post_modified>
            <wp:post_modified_gmt>2015-05-21 19:00:44</wp:post_modified_gmt>
            <wp:comment_status>open</wp:comment_status>
            <wp:ping_status>closed</wp:ping_status>
            <wp:post_name>a-title</wp:post_name>
            <wp:status>publish</wp:status>
            <wp:post_type>post</wp:post_type>
        </item>
        """
        built_file = generate_temporay_file(
            build_xml_stream(xml_items_fragment=fragment).read()
        )

        self.process_import(built_file)

        # One page should be imported
        self.assertEqual(self.importer.imported_pages.count(), 1)
        self.assertEqual(self.importer.imported_pages.first().title, "A title")

        # ItemsCache instance should have 2 writable attributes
        self.assertEqual(len(self.importer.items_cache.__dict__.keys()), 2)

        process_foo.assert_called_once_with(
            self.importer.imported_pages,
            "datatagname",
            getattr(self.importer.items_cache, "foo"),
        )

        process_bar.assert_called_once_with(
            self.importer.imported_pages,
            "datatagname",
            getattr(self.importer.items_cache, "bar"),
        )



from bs4 import BeautifulSoup
from django.conf import settings
from django.utils.module_loading import import_string

from wagtail_wordpress_import.block_builder_defaults import (
    conf_fallback_block,
    conf_html_tags_to_blocks,
)
from wagtail_wordpress_import.prefilters.handle_shortcodes import SHORTCODE_HANDLERS


def conf_promote_child_tags():
    TAGS_TO_PROMOTE = ["iframe", "form", "blockquote"]

    # for each registered shortcode handler add the element_name property
    # to TAGS_TO_PROMOTE
    for handler in SHORTCODE_HANDLERS:
        if handler.is_top_level_html_tag:
            TAGS_TO_PROMOTE.append(handler().element_name)
    return getattr(
        settings,
        "WAGTAIL_WORDPRESS_IMPORTER_PROMOTE_CHILD_TAGS",
        {
            "TAGS_TO_PROMOTE": TAGS_TO_PROMOTE,
            "PARENTS_TO_REMOVE": ["p", "div", "span"],
        },
    )


class BlockBuilder:
    def __init__(self, value, node, logger):
        self.soup = BeautifulSoup(value, "lxml")
        self.blocks = []  # for each page this holds the sequence of StreamBlocks
        self.logged_items = {"processed": 0, "imported": 0, "skipped": 0, "items": []}
        self.node = node
        self.logger = logger

    def promote_child_tags(self):
        """
        Some HTML tags that can be at the top level, e.g. the parent is the
        body when using BS4 are getting placed inside or existed inside other HTML tags.
        We pull out these HTML tags and move them to the top level.
        returns: None
            but modifies the page soup
        """
        config_promote_child_tags = conf_promote_child_tags()
        promotee_tags = config_promote_child_tags["TAGS_TO_PROMOTE"]
        removee_tags = config_promote_child_tags["PARENTS_TO_REMOVE"]

        for promotee in promotee_tags:
            promotees = self.soup.findAll(promotee)
            for promotee in promotees:
                if promotee.parent.name in removee_tags:
                    promotee.parent.replace_with(promotee)

    def get_builder_function(self, element):
        """
        params
            element: an HTML tag
        returns:
            a function to parse the block from configuration
        """

        # Detecting standard blocks to tags
        try:
            builder = conf_html_tags_to_blocks()[element.name]
            return import_string(builder)
        except KeyError:
            pass

        # registered shortcode handlers custom HTML tags
        conf_custom_tags = {}
        for handler in SHORTCODE_HANDLERS:
            cls = handler()
            conf_custom_tags[cls.element_name] = handler

        if element.name in conf_custom_tags:
            handler = conf_custom_tags[element.name]
            # Return the method, so we can call it later like a function that takes a
            # single argument.
            return handler().construct_block

    def build(self):
        """
        params:
            None
        returns:
            a list of block dicts

        The value to be processed here should have only top level HTML tags.
        The HTML is parsed to a sequence of StreamField blocks.
        If a HTML tag does have child blocks we should parse then inside the
        build_block_* method
        """
        soup = self.soup.find("body").findChildren(recursive=False)
        cached_fallback_value = (
            ""  # append fall back content here, by default it's a Rich Text block
        )
        cached_fallback_function = import_string(
            conf_fallback_block()
        )  # Rich Text block
        counter = 0
        for element in soup:  # each single top level tag
            counter += 1

            # the builder function for the element tag from config
            builder_function = self.get_builder_function(element)

            if builder_function:  # build a block
                if cached_fallback_value:
                    cached_fallback_value = cached_fallback_function(
                        cached_fallback_value,
                        self.blocks,
                    )  # before building a block write fall back cache to a block
                self.blocks.append(builder_function(element))
            else:
                if element.text.strip():  # exclude a tag that is empty
                    cached_fallback_value += str(element)

            if cached_fallback_value and counter == len(
                soup
            ):  # the last tag so just build whats left in the fall back cache
                cached_fallback_value = cached_fallback_function(
                    cached_fallback_value, self.blocks
                )

        return self.blocks



import os

from django.test import TestCase, override_settings
from wagtail.core.models import Page
from wagtail_wordpress_import.importers.wordpress import WordpressImporter
from wagtail_wordpress_import.logger import Logger

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"
LOG_DIR = "fakedir"
IMPORTER_RUN_PARAMS_TEST = {
    "app_for_pages": "example",
    "model_for_pages": "TestPage",
    "parent_id": "2",
    "page_types": ["post", "page"],
    "page_statuses": ["publish", "draft"],
}


@override_settings(WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://www.example.com")
class WordpressImporterTests(TestCase):
    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.importer = WordpressImporter(f"{FIXTURES_PATH}/raw_xml.xml")
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages=IMPORTER_RUN_PARAMS_TEST["app_for_pages"],
            model_for_pages=IMPORTER_RUN_PARAMS_TEST["model_for_pages"],
            parent_id=IMPORTER_RUN_PARAMS_TEST["parent_id"],
            page_types=IMPORTER_RUN_PARAMS_TEST["page_types"],
            page_statuses=IMPORTER_RUN_PARAMS_TEST["page_statuses"],
        )

        self.parent_page = Page.objects.get(id=IMPORTER_RUN_PARAMS_TEST["parent_id"])
        self.imported_pages = self.parent_page.get_children().all()
        self.published_pages = self.parent_page.get_children().live()
        self.draft_pages = self.parent_page.get_children().filter(live=False)

    def test_total_pages_imported(self):
        self.assertEqual(len(self.imported_pages), 2)
        self.assertEqual(len(self.published_pages), 1)
        self.assertEqual(len(self.draft_pages), 1)

    def test_page_field_values(self):
        self.assertEqual(self.published_pages.first().title, "Item one title")
        self.assertEqual(self.published_pages.first().slug, "item-one-title")
        self.assertEqual(
            str(self.published_pages.first().first_published_at.date()), "2010-07-13"
        )
        self.assertEqual(
            str(self.published_pages.first().first_published_at.time()), "16:16:46"
        )
        self.assertEqual(
            str(self.published_pages.first().last_published_at.date()), "2010-07-13"
        )
        self.assertEqual(
            str(self.published_pages.first().last_published_at.time()), "16:16:46"
        )
        self.assertEqual(
            str(self.published_pages.first().latest_revision_created_at.date()),
            "2010-07-13",
        )
        self.assertEqual(
            str(self.published_pages.first().latest_revision_created_at.time()),
            "16:16:46",
        )
        self.assertEqual(
            self.published_pages.first().search_description,
            "This page has a default description",
        )

        # debug fields
        self.assertEqual(self.published_pages.first().specific.wp_post_id, 1)
        self.assertEqual(self.published_pages.first().specific.wp_post_type, "post")
        self.assertEqual(
            self.published_pages.first().specific.wp_link,
            "https://www.example.com/item-one-title/",
        )

        ## these fields are only checked for having some content.
        self.assertTrue(self.published_pages.first().specific.body)
        self.assertTrue(self.published_pages.first().specific.wp_raw_content)
        self.assertTrue(self.published_pages.first().specific.wp_block_json)
        self.assertTrue(self.published_pages.first().specific.wp_processed_content)
        ## this field is not used now but is tested as it should be blank
        ## we can remove this field later if we end up not needing it
        self.assertFalse(self.published_pages.first().specific.wp_normalized_styles)

    def test_logger_totals(self):
        processed = self.logger.processed
        self.assertEqual(processed, 5)

        imported = self.logger.imported
        self.assertEqual(imported, 2)

        skipped = self.logger.skipped
        self.assertEqual(skipped, 3)

    def test_logger_lists(self):
        logger = Logger(LOG_DIR)
        logger.items.append(
            {
                "id": 3,
                "title": "Item one title",
                "link": "https://www.example.com/item-one-title/",
                "result": "updated",
                "reason": "existed",
                "datecheck": "",
                "slugcheck": "",
            }
        )
        logger.items.append(
            {
                "id": 4,
                "title": "Item two title",
                "link": "https://www.example.com/item-two-title/",
                "result": "updated",
                "reason": "existed",
                "datecheck": "",
                "slugcheck": "",
            }
        )

        item_id_3 = next(filter(lambda item: item["id"] == 3, logger.items))
        self.assertEqual(item_id_3["title"], "Item one title")

        item_id_4 = next(filter(lambda item: item["id"] == 4, logger.items))
        self.assertEqual(item_id_4["title"], "Item two title")

    def test_page_field_values_with_yoast_plugin_disbaled(self):
        self.assertEqual(
            self.published_pages.first().search_description,
            "This page has a default description",
        )

    def test_page_field_values_if_no_post_date_modified(self):
        page = self.imported_pages.get(title="Item two title")
        print(page)
        self.assertEqual(str(page.first_published_at.date()), "2010-01-13")
        self.assertEqual(str(page.first_published_at.time()), "16:00:00")
        self.assertEqual(str(page.last_published_at.date()), "2010-01-13")
        self.assertEqual(str(page.last_published_at.time()), "16:00:00")
        self.assertEqual(
            str(page.latest_revision_created_at.date()),
            "2010-01-13",
        )
        self.assertEqual(
            str(page.latest_revision_created_at.time()),
            "16:00:00",
        )


IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1 = {
    "app_for_pages": "example",
    "model_for_pages": "TestPage",
    "parent_id": "2",
    "page_types": ["post"],
    "page_statuses": ["publish", "draft"],
}


@override_settings(
    WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_ENABLED=True,
    WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN="http://www.example.com",
)
class WordpressImporterTestsYoastEnabled(TestCase):
    """
    We check here that if the Yoast plugin is enabled import the search description
    from there.
    If the search description is blank or not available then we import the
    <item><description>...</description></item> field.
    If the field is empty then search_description is set as a blank value.
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.importer = WordpressImporter(f"{FIXTURES_PATH}/raw_xml.xml")
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1["app_for_pages"],
            model_for_pages=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1["model_for_pages"],
            parent_id=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1["parent_id"],
            page_types=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1["page_types"],
            page_statuses=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1["page_statuses"],
        )

        self.parent_page = Page.objects.get(
            id=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_1["parent_id"]
        )
        self.imported_pages = self.parent_page.get_children().all()
        self.published_pages = self.parent_page.get_children().live()
        self.draft_pages = self.parent_page.get_children().filter(live=False)

    def test_page_field_values_with_yoast_plugin_enabled(self):
        self.assertEqual(
            self.published_pages.first().search_description,
            "a search description from yoast for Item one",
        )


IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2 = {
    "app_for_pages": "example",
    "model_for_pages": "TestPage",
    "parent_id": "2",
    "page_types": ["testpostmeta"],
    "page_statuses": ["publish", "draft"],
}


@override_settings(WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_ENABLED=True)
class WordpressImporterTestsYoastEnabledMissingTag(TestCase):
    """
    This tests when the expected config for Yoast is different from the
    package default which defaults to use the
    <item><description>...</description></item> field.
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.importer = WordpressImporter(f"{FIXTURES_PATH}/raw_xml.xml")
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2["app_for_pages"],
            model_for_pages=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2["model_for_pages"],
            parent_id=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2["parent_id"],
            page_types=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2["page_types"],
            page_statuses=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2["page_statuses"],
        )

        self.parent_page = Page.objects.get(
            id=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_2["parent_id"]
        )
        self.imported_pages = self.parent_page.get_children().all()
        self.published_pages = self.parent_page.get_children().live()
        self.draft_pages = self.parent_page.get_children().filter(live=False)

    def test_page_field_values_with_yoast_plugin_enabled(self):
        self.assertEqual(
            self.published_pages.first().search_description,
            "This page has a default description",
        )


IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3 = {
    "app_for_pages": "example",
    "model_for_pages": "TestPage",
    "parent_id": "2",
    "page_types": ["postmetachanged"],
    "page_statuses": ["publish", "draft"],
}


@override_settings(
    WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_ENABLED=True,
    WAGTAIL_WORDPRESS_IMPORT_YOAST_PLUGIN_MAPPING={
        "xml_item_key": "wp:postmeta",
        "description_key": "wp:meta_key",
        "description_value": "wp:meta_value",
        "description_key_value": "metadescription",
    },
)
class WordpressImporterTestsYoastEnabledChangedKey(TestCase):
    """
    This tests a different configuration for Yoast.
    The key for the search description is not the same as the package default.
    If the search description is blank or not available then we import the
    <item><description>...</description></item> field.
    If the field is empty then search_description is set as a blank value.
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def setUp(self):
        self.importer = WordpressImporter(f"{FIXTURES_PATH}/raw_xml.xml")
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3["app_for_pages"],
            model_for_pages=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3["model_for_pages"],
            parent_id=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3["parent_id"],
            page_types=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3["page_types"],
            page_statuses=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3["page_statuses"],
        )

        self.parent_page = Page.objects.get(
            id=IMPORTER_RUN_PARAMS_TEST_OVERRIDE_3["parent_id"]
        )
        self.imported_pages = self.parent_page.get_children().all()
        self.published_pages = self.parent_page.get_children().live()
        self.draft_pages = self.parent_page.get_children().filter(live=False)

    def test_page_field_values_with_yoast_plugin_enabled(self):
        self.assertEqual(
            self.published_pages.first().search_description,
            "a search description from yoast using a different key",
        )



from bleach.sanitizer import Cleaner
from django.utils.module_loading import import_string
from wagtail_wordpress_import.prefilters.handle_shortcodes import SHORTCODE_HANDLERS


def filter_bleach_clean(html, options=None):
    """
    We do a final clean up on the processed html to be on the safe side.
    """

    CONF_ALLOWED_TAGS = ALLOWED_TAGS
    if options and options.get("ADDITIONAL_ALLOWED_TAGS"):
        CONF_ALLOWED_TAGS += options["ADDITIONAL_ALLOWED_TAGS"]

    # Registered shortcode handlers generate custom tags
    # so they need to be added to ALLOWED_TAGS

    for handler in SHORTCODE_HANDLERS:
        if handler.is_top_level_html_tag:
            CONF_ALLOWED_TAGS.append(handler().element_name)

    CONF_ALLOWED_ATTRIBUTES = ALLOWED_ATTRIBUTES
    if options and options.get("ADDITIONAL_ALLOWED_ATTRIBUTES"):
        CONF_ALLOWED_ATTRIBUTES.update(options["ADDITIONAL_ALLOWED_ATTRIBUTES"])

    CONF_ALLOWED_STYLES = ALLOWED_STYLES
    if options and options.get("ADDITIONAL_ALLOWED_STYLES"):
        CONF_ALLOWED_STYLES += options["ADDITIONAL_ALLOWED_STYLES"]

    cleaned = Cleaner(
        tags=CONF_ALLOWED_TAGS,
        attributes=CONF_ALLOWED_ATTRIBUTES,
        styles=CONF_ALLOWED_STYLES,
    )

    cleaned_html = cleaned.clean(html)
    return cleaned_html


ALLOWED_TAGS = [
    "a",
    "abbr",
    "acronym",
    "aside",
    "b",
    "blockquote",
    "br",
    "button",
    "caption",
    "center",  # not stictly allowed but we convert id later to css on a div
    "code",
    "col",
    "colgroup",
    "del",
    "div",
    "em",
    "footer",
    "form",
    "figure",
    "figcaption",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "hr",
    "i",
    "iframe",
    "img",
    "input",
    "label",
    "li",
    "ol",
    "option",
    "p",
    "s",
    "script",
    "select",
    "small",
    "span",
    "strike",
    "strong",
    "style",
    "sup",
    "table",
    "tbody",
    "td",
    "th",
    "thead",
    "time",
    "tr",
    "u",
    "ul",
    "wbr",
]

ALLOWED_ATTRIBUTES = {
    "*": ["id"],
    "a": [
        "aria-label",
        "data-wplink-edit",
        "href",
        "rel",
        "target",
        "title",
        "data-uw-rm-brl",
        "data-saferedirecturl",
        "name",
    ],
    "abbr": ["title"],
    "acronym": ["title"],
    "blockquote": [
        "data-cards",
        "data-lang",
        "data-instgrm-permalink",
        "data-instgrm-version",
        "data-instgrm-captioned",
    ],
    "button": ["data-target", "data-toggle"],
    "col": ["width"],
    "div": [
        "title",
        "data-campaign",
        "data-widget-id",
        "data-offer-id",
        "data-aff-id",
        "data-sub-id",
        "data-color",
        "role",
        "data-mm-rates",
    ],
    "form": ["action", "name", "method", "accept-charset"],
    "h2": ["big", "data-toctitle", "data-tocskip", "data-toskip", "data-toc-title"],
    "h3": ["data-toctitle", "data-tocskip"],
    "i": [
        "aria-hidden",
        "data-toggle",
        "data-placement",
        "title",
        "data-original-title",
    ],
    "iframe": [
        "allowfullscreen",
        "src",
        "width",
        "height",
        "frameborder",
        "scrolling",
        "marginwidth",
        "marginheight",
        "data-mce-fragment",
        "data-name",
        "data-link",
    ],
    "img": [
        "alt",
        "border",
        "data-src",
        "height",
        "sizes",
        "src",
        "srcset",
        "title",
        "width",
    ],
    "input": [
        "name",
        "placeholder",
        "readonly",
        "required",
        "type",
        "min",
        "max",
        "step",
        "value",
        "data-type",
    ],
    "label": ["for"],
    "li": ["aria-level", "value", "data-uw-node-idx"],
    "ol": ["data-mm-title", "start"],
    "option": ["data-desc", "data-option", "value", "selected"],
    "p": ["dir", "lang", "data-tocskip", "data-toctitle", "data-mm-rates"],
    "script": ["async", "charset", "defer", "src", "type"],
    "select": ["name"],
    "span": [
        "aria-invalid",
        "data-reactid",
        "data-preserver-spaces",
        "data-mm-rates",
    ],
    "style": ["*"],
    "table": ["width", "dir", "border", "cellspacing", "cellpadding", "summary"],
    "tbody": ["data-mm-rates"],
    "td": [
        "data-label",
        "width",
        "colspan",
        "data-sheets-value",
        "data-sheets-numberformat",
        "data-sheets-formula",
        "data-sheets-hyperlink",
        "rowspan",
    ],
    "th": ["width", "scope", "colspan", "rowspan"],
    "time": ["datetime"],
    "ul": ["data-mm-title", "data-mm-rates"],
}

ALLOWED_STYLES = [
    "font-size",
    "font-weight",
    "font-style: italic;",
    "font-style",
    "text-align: center",
    "text-align: center;",
]



# Generated by Django 3.2.7 on 2021-11-16 10:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0004_alter_body_stream_feild'),
    ]

    operations = [
        migrations.AddField(
            model_name='testpage',
            name='wp_post_meta',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



import os

from django.conf import settings
from django.core.management.base import BaseCommand
from wagtail_wordpress_import.importers.wordpress import WordpressImporter
from wagtail_wordpress_import.logger import Logger

LOG_DIR = "log"


class Command(BaseCommand):
    help = """Run the import process on all items in the XML file and make
    them child pages of a specific page."""

    """
    ./manage.py import_xml path/to/xml/file.xml parent_page_id [--app] [--model] [--type] [--status]

    The default is:
    Import all `post` and `page` types of status `draft` and `publish` as children
    of the page with id of parent_id
    """

    def add_arguments(self, parser):
        parser.add_argument("xml_file", type=str, help="The full path to your xml file")
        parser.add_argument(
            "parent_id",
            type=int,
            help="The page ID of the parent page to use when creating imported pages",
        )
        parser.add_argument(
            "-a",
            "--app",
            type=str,
            help="The app which contains your page models for the import",
            default="pages",
        )
        parser.add_argument(
            "-m",
            "--model",
            type=str,
            help="The page model to use for the imported pages",
            default="PostPage",
        )
        parser.add_argument(
            "-t",
            "--type",
            type=str,
            help="The wordpress post type/s to import. Use a comma to separate multiple types",
            default="page,post",
        )
        parser.add_argument(
            "-s",
            "--status",
            type=str,
            help="The wordpress post statuse/s to import. Use a comma to separate multiple types",
            default="publish,draft",
        )

    def handle(self, **options):
        if not getattr(settings, "WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN", ""):
            self.stdout.write(
                self.style.ERROR(
                    "WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN: is missing in your site settings"
                )
            )
            exit()
        xml_file_path = self.get_xml_file(f"{options['xml_file']}")
        logger = Logger(LOG_DIR)
        importer = WordpressImporter(xml_file_path)
        importer.run(
            page_types=options["type"].split(","),
            page_statuses=options["status"].split(","),
            app_for_pages=options["app"],
            model_for_pages=options["model"],
            parent_id=options["parent_id"],
            logger=logger,
        )
        logger.output_import_summary()
        logger.save_csv_import_report()
        logger.save_csv_images_report()
        logger.save_csv_pagelink_errors_report()

    def get_xml_file(self, xml_file):
        if os.path.exists(xml_file):
            return xml_file

        self.stdout.write(
            self.style.ERROR(f"The xml file cannot be found at: {xml_file}")
        )
        exit()



from django.test import TestCase
from wagtail_wordpress_import.test.tests.utility_functions import (
    mock_image,
    mock_pdf,
    get_soup,
)


class TestFixtures(TestCase):
    def setUp(self):
        self.image = mock_image()
        self.pdf = mock_pdf()

    def test_mock_image(self):
        self.assertEqual(self.image.name, "test.jpg")

    def test_mock_image_content(self):
        image_content = self.image.read()
        self.assertTrue(image_content.startswith(b"\xff\xd8"))

    def test_mock_pdf(self):
        pdf_content = mock_pdf().read()
        self.assertEqual(pdf_content, b"PDF Document")

    def test_get_soup_html_parser(self):
        soup = get_soup("<p>Hello</p>", "html.parser")
        self.assertEqual(soup.p.text, "Hello")

    def test_get_soup_html5lib(self):
        soup = get_soup("<p>Hello</p>", "html5lib")
        self.assertEqual(soup.p.text, "Hello")

    def test_get_soup_lxml(self):
        soup = get_soup("<p>Hello</p>", "lxml")
        self.assertEqual(soup.p.text, "Hello")



from django.conf import settings


def transform_style_bold(soup, tag):
    """
    Replace the input tag name with `b` and remove style attr
    """
    new_tag = soup.new_tag("b")
    new_tag.string = tag.text
    tag.replace_with(new_tag)


def transform_style_italic(soup, tag):
    """
    Replace the input tag name with `i` and remove style attr
    """
    new_tag = soup.new_tag("i")
    new_tag.string = tag.text
    tag.append(new_tag)


def transform_style_bold_italic(soup, tag):
    """
    Replace the input tag name with `b` and a child of `i` and remove style attr
    """
    new_b_tag = soup.new_tag("b")
    new_i_tag = soup.new_tag("i")
    new_i_tag.string = tag.text
    new_b_tag.append(new_i_tag)
    tag.replace_with(new_b_tag)


def transform_style_center(soup, tag):
    """
    apply a new css class to any existing classes
    """
    if tag.attrs.get("class"):
        tag.attrs["class"].append("align-center")
    else:
        tag.attrs["class"] = "align-center"


def transform_style_left(soup, tag):
    """
    apply a new css class to any existing classes
    """
    if tag.attrs.get("class"):
        tag.attrs["class"].append("align-left")
    else:
        tag.attrs["class"] = "align-left"


def transform_style_right(soup, tag):
    """
    apply a new css class to any existing classes
    """
    if tag.attrs.get("class"):
        tag.attrs["class"].append("align-right")
    else:
        tag.attrs["class"] = "align-right"


def transform_float_left(soup, tag):
    """
    apply a new css class to any existing classes
    """
    if tag.attrs.get("class"):
        tag.attrs["class"].append("float-left")
    else:
        tag.attrs["class"] = "float-left"


def transform_float_right(soup, tag):
    """
    apply a new css class to any existing classes
    """
    if tag.attrs.get("class"):
        tag.attrs["class"].append("float-right")
    else:
        tag.attrs["class"] = "float-right"


def transform_html_tag_strong(soup, tag):
    """
    replace the input tag name of `strong` with `b`
    """
    tag.name = "b"


def transform_html_tag_em(soup, tag):
    """
    replace the input tag name of `em` with `i`
    """
    tag.name = "i"


def conf_transform_html_tags_mapping():
    """
    It's intended that a developer can override this configuration and provide their own tag rules

    # example WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING config in your own settings is below

    WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING [
        ("strong", transform_html_tag_strong),
        ("em", transform_html_tag_em),
    ]

    The tag filters can be listed in any order.
    If you don't need a tag filter to run you can remove it from the list.
    If you need another filter to run you can include it in the list and will need
    to provide the filter method in your own wagtail site

    See the documentation in this repo for further help with creating tag filters
    """
    return getattr(
        settings,
        "WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING",
        [
            ("strong", transform_html_tag_strong),
            ("em", transform_html_tag_em),
        ],
    )


def conf_transform_html_tags_enabled():
    """
    In your own settings you can disable the transformation of HTML tags using

    WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_ENABLED = False
    """
    return getattr(
        settings, "WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_ENABLED", True
    )


HTML_TAGS = [
    "address",
    "article",
    "aside",
    "blockquote",
    "canvas",
    "dd",
    "div",
    "dl",
    "dt",
    "fieldset",
    "figcaption",
    "figure",
    "footer",
    "form",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "header",
    "hr",
    "li",
    "main",
    "nav",
    "noscript",
    "ol",
    "p",
    "pre",
    "section",
    "table",
    "tfoot",
    "ul",
    "video",
    "a",
    "abbr",
    "acronym",
    "b",
    "bdo",
    "big",
    "br",
    "button",
    "center",
    "cite",
    "code",
    "dfn",
    "em",
    "i",
    "img",
    "input",
    "kbd",
    "label",
    "map",
    "object",
    "output",
    "q",
    "samp",
    "script",
    "select",
    "small",
    "span",
    "strong",
    "sub",
    "sup",
    "textarea",
    "time",
    "tt",
    "var",
]



from io import StringIO
from django.test import TestCase
from wagtail_wordpress_import.test.tests.xml_boilerplate import (
    xml_stream_footer,
    xml_stream_header,
    build_xml_stream,
)


class TestXmlStream(TestCase):
    def test_building_xml_stream(self):
        """This is just a test that our in-class fixture builder works okay.
        This unit test is a bit sensitive of blank lines and indentation, but the rest
        don't really matter so long as we format valid XML.
        """
        fragment_tags = """
                <wp:foo>
                    <wp:foo_first_name>J.</wp:foo_first_name>
                    <wp:foo_last_name>Money</wp:foo_last_name>
                </wp:foo>"""
        fragment_items = """"""
        built = build_xml_stream(
            xml_tags_fragment=fragment_tags, xml_items_fragment=fragment_items
        ).read()
        expected = StringIO(
            xml_stream_header
            + """
                <wp:foo>
                    <wp:foo_first_name>J.</wp:foo_first_name>
                    <wp:foo_last_name>Money</wp:foo_last_name>
                </wp:foo>"""
            + xml_stream_footer
        ).read()
        self.maxDiff = None
        self.assertEqual(built, expected)



import os
from io import StringIO
from xml.dom import pulldom

from django.conf import settings
from django.test import TestCase, override_settings
from wagtail.core.models import Page
from wagtail_wordpress_import.functions import node_to_dict
from wagtail_wordpress_import.importers.wordpress import WordpressImporter
from wagtail_wordpress_import.logger import Logger
from wagtail_wordpress_import.xml_boilerplate import (
    build_xml_stream,
    generate_temporary_file,
)

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"
LOG_DIR = "fakedir"


class TestImportHooksOverrideXmlTagConfig(TestCase):
    """The overridden conf should be return by import_hooks_xml_tags_to_cache"""

    @override_settings(
        WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE={
            "wp:foo": {"DATA_TAG": "a:tag", "FUNCTION": "path.to.function"},
            "wp:bar": {"DATA_TAG": "a:tag", "FUNCTION": "path.to.function"},
        }
    )
    def test_persist_tags_config(self):
        self.assertEqual(
            getattr(settings, "WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE", {}),
            {
                "wp:foo": {"DATA_TAG": "a:tag", "FUNCTION": "path.to.function"},
                "wp:bar": {"DATA_TAG": "a:tag", "FUNCTION": "path.to.function"},
            },
        )


@override_settings(
    WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE={
        "wp:foo": {"DATA_TAG": "not:required", "FUNCTION": "path.to.function"},
        "wp:bar": {"DATA_TAG": "not:required", "FUNCTION": "path.to.function"},
    }
)
class TestImportHooksXmlTagsPersisted(TestCase):
    """The overridden config should result in TagsCache instance containing two
    dicts with keys foo and bar. Each dict should contain all XML tags and
    values from the fragment"""

    def setUp(self):
        self.logger = Logger("foo")

    def process_item(self, xml_stream):
        """Turn an XML stream into a node dict
        We expect there to be only one node in the XML stream.
        This is duplicating a bunch of code from WorpressImporter. To make this a better
        unit test, we should instead make WordpressImporter more patchable, and less
        reliant on detecting app, Page IDs and logging configuration within its run
        method.
        """
        self.importer = WordpressImporter(xml_stream)
        xml_doc = pulldom.parse(xml_stream)
        for event, node in xml_doc:
            if event == pulldom.START_ELEMENT and node.tagName in getattr(
                settings, "WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE", {}
            ):
                xml_doc.expandNode(node)
                item = node_to_dict(node)
                self.importer.tags_cache.add_item_to_cache(node.tagName, item)

        return self.importer.tags_cache

    def test_xml_tags_are_cached_without_duplicates(self):
        fragment = """
        <wp:foo>
            <wp:foo_id>3</wp:foo_id>
            <wp:foo_login>rickw</wp:foo_login>
            <wp:foo_email>rick@example.com</wp:foo_email>
            <wp:foo_display_name>Rick Wakeman</wp:foo_display_name>
            <wp:foo_first_name>Rick</wp:foo_first_name>
            <wp:foo_last_name>Wakeman</wp:foo_last_name>
        </wp:foo>
        <wp:bar>
            <wp:bar_id>12345</wp:bar_id>
            <wp:bar_nicename>some-stuff</wp:bar_nicename>
        </wp:bar>
        <wp:bar>
            <wp:bar_id>12345</wp:bar_id>
            <wp:bar_nicename>some-stuff</wp:bar_nicename>
        </wp:bar>
        """
        built = build_xml_stream(xml_tags_fragment=fragment).read()
        cache = self.process_item(StringIO(built))

        # using getattr here because then keys can contain special characters
        foo = getattr(cache, "wp:foo")
        self.assertIsInstance(foo, list)
        self.assertEqual(len(foo), 1)
        self.assertIsInstance(foo[0], dict)

        # tests for first item in the list
        self.assertEqual(foo[0]["wp:foo_id"], 3)
        self.assertEqual(foo[0]["wp:foo_login"], "rickw")
        self.assertEqual(foo[0]["wp:foo_email"], "rick@example.com")
        self.assertEqual(foo[0]["wp:foo_display_name"], "Rick Wakeman")
        self.assertEqual(foo[0]["wp:foo_first_name"], "Rick")
        self.assertEqual(foo[0]["wp:foo_last_name"], "Wakeman")

        bar = getattr(cache, "wp:bar")
        self.assertIsInstance(bar, list)
        self.assertEqual(len(bar), 1)
        self.assertIsInstance(bar[0], dict)

        # values of the second item in the list
        self.assertEqual(bar[0]["wp:bar_id"], 12345)
        self.assertEqual(bar[0]["wp:bar_nicename"], "some-stuff")

        # the second item in the fixture <wp:bar> should not be cached
        with self.assertRaises(IndexError) as raises:
            error = bar[1]
        self.assertEqual(str(raises.exception), "list index out of range")


class WordpressImporterTestsCheckXmlTagsNotCached(TestCase):
    """
    This tests that the XML tags are not cached during an import because there
    is no settings.WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE
    """

    fixtures = [
        f"{FIXTURES_PATH}/dump.json",
    ]

    def process_import(self, xml_stream):
        self.importer = WordpressImporter(xml_stream)
        self.logger = Logger(LOG_DIR)
        self.importer.run(
            logger=self.logger,
            app_for_pages="example",
            model_for_pages="TestPage",
            parent_id=2,
            page_types=["post"],
            page_statuses=["publish"],
        )
        self.parent_page = Page.objects.get(id=2)
        self.imported_pages = self.parent_page.get_children().all()

    def test_xml_tags_not_cached(self):
        fragment = """
        <wp:foo>
            <wp:foo_id>3</wp:foo_id>
            <wp:foo_login>rickw</wp:foo_login>
            <wp:foo_email>rick@example.com</wp:foo_email>
            <wp:foo_display_name>Rick Wakeman</wp:foo_display_name>
            <wp:foo_first_name>Rick</wp:foo_first_name>
            <wp:foo_last_name>Wakeman</wp:foo_last_name>
        </wp:foo>
        <item>
            <title>A title</title>
            <link>https://www.example.com/a-title</link>
            <description />
            <content:encoded />
            <excerpt:encoded />
            <wp:post_id>44221</wp:post_id>
            <wp:post_date>2015-05-21 15:00:31</wp:post_date>
            <wp:post_date_gmt>2015-05-21 19:00:31</wp:post_date_gmt>
            <wp:post_modified>2015-05-21 15:00:44</wp:post_modified>
            <wp:post_modified_gmt>2015-05-21 19:00:44</wp:post_modified_gmt>
            <wp:comment_status>open</wp:comment_status>
            <wp:ping_status>closed</wp:ping_status>
            <wp:post_name>a-title</wp:post_name>
            <wp:status>publish</wp:status>
            <wp:post_type>post</wp:post_type>
        </item>
        """
        built_file = generate_temporary_file(
            build_xml_stream(xml_tags_fragment=fragment).read()
        )

        self.process_import(built_file)

        # there should be one page imported
        self.assertEqual(self.imported_pages.count(), 1)
        self.assertEqual(self.imported_pages.first().title, "A title")

        # assertions for the content of tags_cache
        self.assertEqual(len(self.importer.tags_cache.__dict__.keys()), 0)



from .bleach_filter import filter_bleach_clean as bleach_clean  # NOQA
from .transform_styles_filter import (
    filter_transform_inline_styles as transform_inline_styles,
)  # NOQA
from .linebreaks_wp_filter import filter_linebreaks_wp as linebreaks_wp  # NOQA
from .handle_shortcodes import (
    filter_transform_shortcodes as transform_shortcodes,
)  # NOQA



from django.test import TestCase
from wagtail_wordpress_import.functions import get_attr_as_list, snakecase_key


class TestSnakeCaseKey(TestCase):
    def test_snake_case_key_remove_colon(self):
        self.assertEqual(snakecase_key("foo:bar"), "foo_bar")

    def test_snake_case_key_remove_leading_underscore(self):
        self.assertEqual(snakecase_key("_foo_bar"), "foo_bar")


class TestDictToList(TestCase):
    def test_default_with_string(self):
        node = "foo"
        self.assertEqual(get_attr_as_list(node, "foo"), [])

    def test_default_with_dict(self):
        node = {}
        self.assertEqual(get_attr_as_list(node, "foo"), [])

    def test_default_with_list(self):
        node = []
        self.assertEqual(get_attr_as_list(node, "foo"), [])

    def test_default_list(self):
        node = ["foo"]
        self.assertEqual(get_attr_as_list(node, "foo"), [])

    def test_default_empty_dict(self):
        node = {"foo": {}}
        self.assertEqual(get_attr_as_list(node, "foo"), [])

    def test_with_expected_dict(self):
        node = {"foo": {"bar": "baz", "baz": "bar"}}
        self.assertEqual(get_attr_as_list(node, "foo"), [{"bar": "baz", "baz": "bar"}])

    def test_with_expected_list(self):
        node = {"foo": [{"bar": "baz", "baz": "bar"}]}
        self.assertEqual(get_attr_as_list(node, "foo"), [{"bar": "baz", "baz": "bar"}])



import shortcodes


def find_all_shortcodes(text):
    found_shortcodes = []

    if "[" in text:
        lexer = shortcodes.Lexer(text, "[", "]", "\\[")
        try:
            for token in lexer.tokenize():
                if token.type == "TAG":
                    shortcode_name = token.text.split(" ")[0]

                    # Ignore closing shortcodes
                    if not shortcode_name.startswith("/"):
                        found_shortcodes.append(shortcode_name)
        except shortcodes.ShortcodeSyntaxError:
            return []

    return found_shortcodes



from django.apps import AppConfig

class WagtailWordpressImportAppConfig(AppConfig):
    label = "wagtail_wordpress_import"
    name = "wagtail_wordpress_import"
    verbose_name = "Wagtail wordpress import"



import os

from bs4 import BeautifulSoup
from django.test import TestCase
from wagtail_wordpress_import.prefilters.linebreaks_wp_filter import (
    filter_linebreaks_wp,
)

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"


class TestLinebreaks(TestCase):
    def setUp(self):
        self.raw_html_file = open(f"{FIXTURES_PATH}/raw_text.txt", "r")
        self.stream = self.raw_html_file.read()
        self.soup = BeautifulSoup(filter_linebreaks_wp(self.stream), "html.parser")

    def test_linebreaks_wp(self):
        p_tags = self.soup.findAll("p")
        self.assertEqual(len(p_tags), 7)

    def test_simple_string(self):
        input = """line 1

        line 2

        line 3"""
        soup = BeautifulSoup(filter_linebreaks_wp(input), "html.parser")
        paragraphs = soup.findAll("p")

        self.assertEqual(len(paragraphs), 3)
        for i in range(len(paragraphs)):
            self.assertEqual(paragraphs[i].text.strip()[-1], str(i + 1))



import copy
import json
from datetime import datetime
from functools import cached_property
from xml.dom import pulldom

from bs4 import BeautifulSoup
from django.apps import apps
from django.conf import settings
from django.utils.module_loading import import_string
from django.utils.text import slugify
from django.utils.timezone import make_aware
from wagtail.core.models import Page
from wagtail_wordpress_import.block_builder import BlockBuilder
from wagtail_wordpress_import.functions import (
    get_attr_as_list,
    node_to_dict,
    snakecase_key,
)
from wagtail_wordpress_import.importers.import_hooks import ItemsCache, TagsCache
from wagtail_wordpress_import.importers.wordpress_defaults import (
    category_name_min_length,
    category_plugin_enabled,
    debug_enabled,
    get_category_model,
    yoast_plugin_config,
    yoast_plugin_enabled,
)
from wagtail_wordpress_import.prefilters.linebreaks_wp_filter import (
    filter_linebreaks_wp,
)

DEFAULT_PREFILTERS = [
    {
        "FUNCTION": "wagtail_wordpress_import.prefilters.linebreaks_wp",
    },
    {
        "FUNCTION": "wagtail_wordpress_import.prefilters.transform_shortcodes",
    },
    {
        "FUNCTION": "wagtail_wordpress_import.prefilters.transform_inline_styles",
    },
    {
        "FUNCTION": "wagtail_wordpress_import.prefilters.bleach_clean",
    },
]


class WordpressImporter:
    def __init__(self, xml_file_path):
        self.xml_file = xml_file_path
        self.imported_page_ids = []
        self.page_link_errors = []
        self.items_cache = ItemsCache()
        self.tags_cache = TagsCache()

    def run(self, *args, **kwargs):
        self.logger = kwargs["logger"]
        xml_doc = pulldom.parse(self.xml_file)

        try:
            self.page_model_class = apps.get_model(
                kwargs["app_for_pages"], kwargs["model_for_pages"]
            )
        except LookupError:
            print(
                f"The app `{kwargs['app_for_pages']}` and/or page model `{kwargs['model_for_pages']}` cannot be found!"
            )
            print(
                "Check the command line options -a and -m match an existing Wagtail app and Wagtail page model"
            )
            exit()

        try:
            self.parent_page_obj = Page.objects.get(pk=kwargs["parent_id"])
        except Page.DoesNotExist:
            print(f"A page with id {kwargs['parent_id']} does not exist")
            exit()

        for event, node in xml_doc:
            """
            Each node represents a tag in the xml.
            `event` is true for a start element.
            """

            if event == pulldom.START_ELEMENT and node.tagName in getattr(
                settings, "WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE", {}
            ):  # add top level XML tags to cache
                xml_doc.expandNode(node)
                item = node_to_dict(node)
                self.tags_cache.add_item_to_cache(node.tagName, item)

            if event == pulldom.START_ELEMENT and node.tagName == "item":
                xml_doc.expandNode(node)
                item = node_to_dict(node)
                self.logger.processed += 1

                post_type = item.get("wp:post_type")
                if post_type in getattr(
                    settings, "WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE", {}
                ):  # add item level XML tags to cache
                    self.items_cache.add_item_to_cache(post_type, item)

                if (
                    item.get("wp:post_type") in kwargs["page_types"]
                    and item.get("wp:status") in kwargs["page_statuses"]
                ):

                    wordpress_item = WordpressItem(item, self.logger)

                    try:
                        page = self.page_model_class.objects.get(
                            wp_post_id=wordpress_item.cleaned_data.get("wp_post_id")
                        )
                    except self.page_model_class.DoesNotExist:
                        page = self.page_model_class()

                    # add categories for this page if categories plugin is enabled
                    if category_plugin_enabled() and get_category_model():
                        self.connect_page_categories(
                            page, import_string(get_category_model()), item
                        )

                    cleaned_data = wordpress_item.cleaned_data

                    body = cleaned_data.get("body")

                    self.check_stream_field_block_types(
                        page, body
                    )  # if the body streamfield is invalid, exit with a ValueError

                    page.import_wordpress_data(cleaned_data)

                    if item.get("wp:status") == "draft":
                        setattr(page, "live", False)
                    else:
                        setattr(page, "live", True)

                    if page.id:
                        page.save()
                        self.logger.imported += 1
                        self.logger.items.append(
                            {
                                "id": page.id,
                                "title": page.title,
                                "link": item.get("link"),
                                "result": "updated",
                                "reason": "existed",
                                "datecheck": wordpress_item.date_changed,
                                "slugcheck": wordpress_item.slug_changed,
                            }
                        )
                    else:
                        self.parent_page_obj.add_child(instance=page)
                        page.save()
                        self.logger.imported += 1
                        self.logger.items.append(
                            {
                                "id": page.id,
                                "title": page.title,
                                "link": item.get("link"),
                                "result": "created",
                                "reason": "existed",
                                "datecheck": wordpress_item.date_changed,
                                "slugcheck": wordpress_item.slug_changed,
                            }
                        )

                    self.imported_page_ids.append(page.id)

                else:
                    self.logger.skipped += 1
                    self.logger.items.append(
                        {
                            "id": 0,
                            "title": "",
                            "link": "",
                            "result": "excluded",
                            "reason": "not a page type or status to import",
                            "datecheck": "",
                            "slugcheck": "",
                        }
                    )
            else:
                self.logger.items.append(
                    {
                        "id": 0,
                        "title": "",
                        "link": "",
                        "result": "excluded",
                        "reason": "not a item",
                        "datecheck": "",
                        "slugcheck": "",
                    }
                )
            self.logger.log_progress()

        self.imported_pages = self.page_model_class.objects.filter(
            id__in=[id for id in self.imported_page_ids]
        ).specific()

        self.connect_richtext_page_links(self.imported_pages)

        """Run all hooks in settings.WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE"""
        for hook, actions in getattr(
            settings, "WORDPRESS_IMPORT_HOOKS_ITEMS_TO_CACHE", {}
        ).items():
            import_string(actions["FUNCTION"])(
                self.imported_pages,
                actions["DATA_TAG"],
                getattr(self.items_cache, hook),
            )

        """Run all hooks in settings.WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE"""
        for hook, actions in getattr(
            settings, "WORDPRESS_IMPORT_HOOKS_TAGS_TO_CACHE", {}
        ).items():
            import_string(actions["FUNCTION"])(
                self.imported_pages,
                actions["DATA_TAG"],
                getattr(self.tags_cache, hook),
            )

    @staticmethod
    def check_stream_field_block_types(page, body):
        """Body JSON is validated to check it is using only StreamField blocks declared in the model StreamField
        An exception will stop the import process and display the ValueError
        """
        body = json.loads(body)
        item_block_types = [item["type"] for item in body]
        page_stream_types = " ".join(
            [stream_block for stream_block in page.body.stream_block.child_blocks]
        )

        for item_block in item_block_types:
            if item_block not in page_stream_types:
                raise ValueError(f"Invalid page streamfield block types: {item_block}")

    def analyze_html(self, html_analyzer, *, page_types, page_statuses):
        xml_doc = pulldom.parse(self.xml_file)

        for event, node in xml_doc:
            """
            Each node represents a tag in the xml.
            `event` is true for a start element.
            """
            if event == pulldom.START_ELEMENT and node.tagName == "item":
                xml_doc.expandNode(node)
                item = node_to_dict(node)
                if (
                    item.get("wp:post_type") in page_types
                    and item.get("wp:status") in page_statuses
                ):

                    html_analyzer.analyze(
                        filter_linebreaks_wp(item.get("content:encoded"))
                    )

    def connect_richtext_page_links(self, imported_pages):
        """
        Update the StreamField content of each page by reconstructing
        the existing blocks. Only update rich_text blocks by analysing the anchor links
        in update_rich_text_page_links()
        """
        for page in imported_pages:
            stream_data = page.body._raw_data
            reconstructed_blocks = []
            for block in stream_data:
                del block["id"]  # these get created on save so lose the old id
                if block["type"] == "rich_text":
                    block["value"] = str(self.update_rich_text_page_links(block, page))
                reconstructed_blocks.append(block)
            page.body = json.dumps(stream_data)
            page.save()

    def update_rich_text_page_links(self, block, page):
        """
        Get all the anchor tags (bs4) and replace the anchor tag with
        <a id="{wagtail page id}" linktype="page">{the current anchor text}</a>
        If a wagtail page cannot be found for the anchor link, ignore it but
        save to the log.
        """
        soup = BeautifulSoup(block["value"], "html.parser")
        links = soup.findAll("a")

        for link in links:
            page_link = self.get_page(link.attrs.get("href"), page)
            if page_link:
                new_tag = soup.new_tag("a")
                new_tag.attrs["id"] = page_link.id
                new_tag.attrs["linktype"] = "page"
                new_tag.string = link.text
                link.replace_with(new_tag)
        return soup

    def get_page(self, link, page):
        try:
            if debug_enabled():
                self.logger.page_link_errors.append((link, page))
            return self.page_model_class.objects.get(wp_link=link)
        except self.page_model_class.DoesNotExist:
            pass

    def connect_page_categories(self, page, category_model, item):
        if "category" in item.keys():
            categories = [
                category
                for category in item["category"]
                if category and len(category) > category_name_min_length()
            ]

            page_categories = []

            for name in categories:
                category, created = category_model.objects.get_or_create(name=name)
                page_categories.append(category)

            page.categories = page_categories


def default_prefilters():
    return [
        {
            "FUNCTION": "wagtail_wordpress_import.prefilters.linebreaks_wp",
        },
        {
            "FUNCTION": "wagtail_wordpress_import.prefilters.transform_shortcodes",
        },
        {
            "FUNCTION": "wagtail_wordpress_import.prefilters.transform_inline_styles",
        },
        {
            "FUNCTION": "wagtail_wordpress_import.prefilters.bleach_clean",
        },
    ]


class WordpressItem:
    def __init__(self, node, logger):
        self.node = node
        self.raw_body = self.node["content:encoded"]
        self.slug_changed = ""
        self.date_changed = ""
        self.image_errors = []

        self.debug_content = {}
        self.logger = logger
        self.post_meta_items = None
        self.post_item_tags = None

    def prefilter_content(self, content):
        """
        FILTERS ARE CUMULATIVE: Each filter receives the output from the previous filter.
        """
        cached_result = content

        for filter in getattr(
            settings, "WAGTAIL_WORDPRESS_IMPORT_PREFILTERS", DEFAULT_PREFILTERS
        ):
            function = import_string(filter["FUNCTION"])
            cached_result = function(cached_result, filter.get("OPTIONS"))
            if debug_enabled():
                self.debug_content[function.__name__] = cached_result

        return cached_result

    def cleaned_title(self):
        title = self.node.get("title", None)
        if title:
            return title.strip()
        else:
            return "no-title-available-{}".format(self.node.get("wp:post_id"))

    def cleaned_slug(self):
        """
        Clean up the slugs from the XML import file
        Some pages have no slug and some have unexpected characters.
        If a slug is not provided create one from page title.
        If a slug is changed it's recorded in the logger
        """
        if not self.node["wp:post_name"]:
            slug = slugify(self.cleaned_title())
            self.slug_changed = "blank slug"  # logging
        else:
            slug = slugify(self.node["wp:post_name"])

        # some slugs have illegal characters so it will be changed
        if slug != self.node["wp:post_name"]:
            self.slug_changed = "slug fixed"  # logging

        return slug

    def cleaned_first_published_at(self):
        return self.clean_date(self.node["wp:post_date_gmt"].strip())

    def cleaned_last_published_at(self):
        try:
            return self.clean_date(self.node["wp:post_modified_gmt"].strip())
        except KeyError:
            return self.cleaned_first_published_at()

    def cleaned_latest_revision_created_at(self):
        try:
            return self.clean_date(self.node["wp:post_modified_gmt"].strip())
        except KeyError:
            return self.cleaned_first_published_at()

    def clean_date(self, value):
        """
        We need a proper date format.
        Some dates are not suitable date strings in the xml, if so return a
        specific date so it can be saved in Wagtail and record it in the logger.
        """

        if value == "0000-00-00 00:00:00":
            # setting this date so it can be found in wagtail admin
            value = "1900-01-01 00:00:00"
            self.date_changed = True

        date_utc = "T".join(value.split(" "))
        date_formatted = make_aware(datetime.strptime(date_utc, "%Y-%m-%dT%H:%M:%S"))

        return date_formatted

    def cleaned_post_id(self):
        return int(self.node["wp:post_id"])

    def cleaned_post_type(self):
        return str(self.node["wp:post_type"].strip())

    def cleaned_link(self):
        return str(self.node["link"].strip())

    def body_stream_field(self, content):
        builder = BlockBuilder(content, self.node, self.logger)
        builder.promote_child_tags()
        blocks_dict = builder.build()
        if debug_enabled():
            self.debug_content["block_json"] = blocks_dict
        return json.dumps(blocks_dict)

    def get_yoast_description_value(self):
        """
        The Wordpress yoast plugin seems to have different fields available on
        a item record and not always contains the _yoast_wpseo_metadesc field.

        This parses the wp:postmeta field to check if a _yoast_wpseo_metadesc
        is available. If not it returns a blank string or the default description field
        from the XML import file <item><description>...</description></item>
        """
        meta_value = None

        if yoast_plugin_config()["xml_item_key"] in self.node.keys():
            meta_items = (
                self.node[yoast_plugin_config()["xml_item_key"]]
                if isinstance(self.node[yoast_plugin_config()["xml_item_key"]], list)
                else [self.node[yoast_plugin_config()["xml_item_key"]]]
            )  # if there is only one wp:post_meta item, it's a dictionary
            # we need a list of dictionaries
            for item in meta_items:
                meta_key_values = list(item.values())
                if meta_key_values[0] == yoast_plugin_config()["description_key_value"]:
                    meta_value = meta_key_values[1] or ""

        if not meta_value:
            meta_value = self.node.get("description") or ""

        return meta_value

    def cleaned_search_description(self):
        if yoast_plugin_enabled():
            return self.get_yoast_description_value().strip()

        if self.node.get("description") is not None:
            return self.node.get("description").strip()
        else:
            return ""

    def clean_wp_post_meta(self):
        """
        The complete XML item node is converted to a dict.
        The XML item node 'content:encoded' is not included in the dict.
        The original XML item node 'wp:postmeta' is not included in the dict.
        The dict item values are all data values without any nesting of 'list'
        or 'dict' types.
        """

        node = copy.deepcopy(self.node)

        # the content:encoded tag is not needed in the
        # wp_post_meta field as the content is parsed elsewhere
        del node["content:encoded"]

        cleaned = {}  # the final value to be returned

        for item in get_attr_as_list(node, "wp:postmeta"):
            cleaned[snakecase_key(item["wp:meta_key"])] = item["wp:meta_value"]

        # Remove wp:postmeta from the node so we don't use it's values again.
        if "wp:postmeta" in node:
            del node["wp:postmeta"]

        for k, v in node.items():
            cleaned[snakecase_key(k)] = v

        return cleaned

    @cached_property
    def cleaned_data(self):
        """Return all processed page model fields"""

        return {
            "title": self.cleaned_title(),
            "slug": self.cleaned_slug(),
            "first_published_at": self.cleaned_first_published_at(),
            "last_published_at": self.cleaned_last_published_at(),
            "latest_revision_created_at": self.cleaned_latest_revision_created_at(),
            "body": self.body_stream_field(self.prefilter_content(self.raw_body)),
            "search_description": self.cleaned_search_description(),
            "wp_post_id": self.cleaned_post_id(),
            "wp_post_type": self.cleaned_post_type(),
            "wp_link": self.cleaned_link(),
            "wp_block_json": self.debug_content.get("block_json"),
            "wp_processed_content": self.debug_content.get(
                "filter_transform_inline_styles"
            ),
            "wp_normalized_styles": "",
            "wp_raw_content": self.debug_content.get("filter_linebreaks_wp"),
            "wp_post_meta": self.clean_wp_post_meta(),
        }



from django.conf.urls import include
from django.contrib import admin
from django.urls import re_path
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.core import urls as wagtail_urls
from wagtail.documents import urls as wagtaildocs_urls

urlpatterns = [
    re_path(r"^django-admin/", admin.site.urls),
    re_path(r"^admin/", include(wagtailadmin_urls)),
    re_path(r"^documents/", include(wagtaildocs_urls)),
    re_path(r"", include(wagtail_urls)),
]



import os

from bs4 import BeautifulSoup
from django.test import TestCase, override_settings
from wagtail_wordpress_import.prefilters.transform_styles_defaults import (
    transform_float_left,
    transform_float_right,
    transform_html_tag_em,
    transform_html_tag_strong,
    transform_style_bold,
    transform_style_center,
    transform_style_italic,
    transform_style_left,
    transform_style_right,
)
from wagtail_wordpress_import.prefilters.transform_styles_filter import (
    filter_transform_inline_styles,
    normalize_style_attrs,
)

BASE_PATH = os.path.dirname(os.path.dirname(__file__))
FIXTURES_PATH = BASE_PATH + "/fixtures"


class TestTransformStylesFilter(TestCase):
    def test_normalize_style_attrs(self):
        raw_html_file = open(f"{FIXTURES_PATH}/raw_html.txt", "r")
        soup = normalize_style_attrs(BeautifulSoup(raw_html_file, "html.parser"))
        span = soup.find("span")

        self.assertEqual(span.attrs["style"], "font-style:italic;font-weight:bold;")

    def test_filter_transform_inline_styles(self):
        input = open(f"{FIXTURES_PATH}/raw_html.txt", "r")
        output = filter_transform_inline_styles(input)
        soup = BeautifulSoup(output, "html.parser")

        """
        This is worthy of a note.
        <span style="font-weight: bold;font-style:italic;">Lorem ipsum (xcounterx) dolor sit amet</span>
        Gets transformed to
        <b><i>Lorem ipsum (xcounterx) dolor sit amet</i></b>
        Because the span has two specific styles.
        """
        first_tag = soup.find("b")
        self.assertEqual(first_tag.name, "b")

        with self.assertRaises(KeyError):
            self.assertTrue(first_tag.attrs["style"])

        first_tag_child = first_tag.find("i")
        self.assertEqual(first_tag_child.name, "i")
        self.assertEqual(first_tag_child.text, "Lorem ipsum (xcounterx) dolor sit amet")

        with self.assertRaises(KeyError):
            self.assertTrue(first_tag_child.attrs["style"])

        heading_tag = soup.find("h2")
        self.assertEqual(heading_tag.name, "h2")

        heading_tag_child = heading_tag.find("b")
        self.assertEqual(heading_tag_child.name, "b")
        self.assertIsNone(heading_tag_child.attrs.get("style"))

    def test_transform_style_bold(self):
        input = '<span style="font-weight: bold;">Text content</span>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_style_bold(soup, soup.find("span"))
        bold = soup.find("b")
        span = soup.find("span")

        self.assertTrue(bold)
        self.assertFalse(span)

    def test_transform_style_italic(self):
        input = '<span style="font-style: italic;">Text content</span>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_style_italic(soup, soup.find("span"))
        italic = soup.find("i")

        self.assertTrue(italic)

    def test_transform_style_center(self):
        input = '<p style="text-align: center;">Text content</p>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_style_center(soup, soup.find("p"))
        paragraph = soup.find("p")

        self.assertTrue(paragraph)
        self.assertIn("align-center", paragraph.attrs["class"])

    def test_transform_style_left(self):
        input = '<p style="text-align: left;">Text content</p>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_style_left(soup, soup.find("p"))
        paragraph = soup.find("p")

        self.assertTrue(paragraph)
        self.assertIn("align-left", paragraph.attrs["class"])

    def test_transform_style_right(self):
        input = '<p style="text-align: right;">Text content</p>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_style_right(soup, soup.find("p"))
        paragraph = soup.find("p")

        self.assertTrue(paragraph)
        self.assertIn("align-right", paragraph.attrs["class"])

    def test_transform_float_left(self):
        input = '<p style="float: left;">Text content</p>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_float_left(soup, soup.find("p"))
        paragraph = soup.find("p")

        self.assertTrue(paragraph)
        self.assertIn("float-left", paragraph.attrs["class"])

    def test_transform_float_right(self):
        input = '<p style="float: right;">Text content</p>'
        soup = normalize_style_attrs(BeautifulSoup(input, "html.parser"))
        transform_float_right(soup, soup.find("p"))
        paragraph = soup.find("p")

        self.assertTrue(paragraph)
        self.assertIn("float-right", paragraph.attrs["class"])

    def test_transform_html_tag_strong(self):
        input = "<strong>Text content</strong>"
        soup = BeautifulSoup(input, "html.parser")
        transform_html_tag_strong(soup, soup.find("strong"))
        strong = soup.find("b")

        self.assertTrue(strong)

    def test_transform_html_tag_em(self):
        input = "<em>Text content</em>"
        soup = BeautifulSoup(input, "html.parser")
        transform_html_tag_em(soup, soup.find("em"))
        italic = soup.find("i")

        self.assertTrue(italic)


def testing_transform_html_tag_blockquote(soup, tag):
    """
    a function to test as if passed in by a developer
    in their own config
    """
    tag.name = "div"


class TestTransformStylesFilterHtmlTagsDeveloperOverrides(TestCase):
    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING=[],
    )
    def test_filter_transform_html_tag_empty(self):
        input = "<em>Text content</em><strong>Text content</strong>"
        output = filter_transform_inline_styles(input)
        soup = BeautifulSoup(output, "html.parser")
        italic = soup.find("em")
        bold = soup.find("strong")

        self.assertTrue(italic)
        self.assertTrue(bold)

    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING=[
            ("strong", transform_html_tag_strong),
        ],
    )
    def test_filter_transform_html_tag_strong(self):
        input = "<em>Text content</em><strong>Text content</strong>"
        output = filter_transform_inline_styles(input)
        soup = BeautifulSoup(output, "html.parser")
        italic = soup.find("em")
        bold = soup.find("b")

        self.assertTrue(italic)
        self.assertTrue(bold)

    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING=[
            ("em", transform_html_tag_em),
        ],
    )
    def test_filter_transform_html_tag_em(self):
        input = "<em>Text content</em><strong>Text content</strong>"
        output = filter_transform_inline_styles(input)
        soup = BeautifulSoup(output, "html.parser")
        italic = soup.find("i")
        bold = soup.find("strong")

        self.assertTrue(italic)
        self.assertTrue(bold)

    @override_settings(
        WAGTAIL_WORDPRESS_IMPORT_TRANSFORM_HTML_TAGS_MAPPING=[
            ("strong", transform_html_tag_strong),
            ("em", transform_html_tag_em),
            ("blockquote", testing_transform_html_tag_blockquote),
        ],
    )
    def test_filter_transform_html_tag_em(self):
        input = "<em>Text content</em><strong>Text content</strong><blockquote>Text content</blockquote>"
        output = filter_transform_inline_styles(input)
        soup = BeautifulSoup(output, "html.parser")
        italic = soup.find("i")
        bold = soup.find("b")
        div = soup.find("div")

        self.assertTrue(italic)
        self.assertTrue(bold)
        self.assertTrue(div)



from io import StringIO
import tempfile

xml_stream_header = StringIO(
    """
    <rss xmlns:content="http://purl.org/rss/1.0/modules/content/" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:excerpt="http://wordpress.org/export/1.2/excerpt/" xmlns:wp="http://wordpress.org/export/1.2/" version="2.0">
    <channel>
        <title>Foo</title>
        <link>https://www.example.com</link>
        <pubDate>Fri, 30 Jul 2021 11:56:01 +0000</pubDate>
        <language>en-US</language>
        <wp:wxr_version>1.2</wp:wxr_version>
        <wp:base_site_url>https://www.budgetsaresexy.com</wp:base_site_url>
        <wp:base_blog_url>https://www.budgetsaresexy.com</wp:base_blog_url>"""
).read()

xml_stream_footer = StringIO(
    """
    </channel>
    </rss>
    """
).read()


def build_xml_stream(xml_tags_fragment="", xml_items_fragment=""):
    """Formats the boilerplate XML template with the provided fragment for top level tags."""

    return StringIO(
        xml_stream_header + xml_tags_fragment + xml_items_fragment + xml_stream_footer
    )


def generate_temporary_file(xml_stream):

    temp_file = tempfile.NamedTemporaryFile(delete=False)

    with open(temp_file.name, "w") as f:
        f.write(xml_stream)

    return temp_file.name



import requests
from bs4 import BeautifulSoup
from django.conf import settings
from django.core.files import File
from django.core.files.temp import NamedTemporaryFile
from django.utils.module_loading import import_string
from wagtail.documents import get_document_model
from wagtail.images import get_image_model

ImportedImage = get_image_model()
ImportedDocument = get_document_model()


def conf_html_tags_to_blocks():
    return getattr(
        settings,
        "WAGTAIL_WORDPRESS_IMPORTER_CONVERT_HTML_TAGS_TO_BLOCKS",
        {
            "h1": "wagtail_wordpress_import.block_builder_defaults.build_heading_block",
            "table": "wagtail_wordpress_import.block_builder_defaults.build_table_block",
            "iframe": "wagtail_wordpress_import.block_builder_defaults.build_iframe_block",
            "form": "wagtail_wordpress_import.block_builder_defaults.build_form_block",
            "img": "wagtail_wordpress_import.block_builder_defaults.build_image_block",
            "blockquote": "wagtail_wordpress_import.block_builder_defaults.build_block_quote_block",
        },
    )


def fetch_url(src, allow_redirects=True):
    """general purpose url fetcher with ability to pass in own config"""
    try:
        response = requests.get(
            src,
            **getattr(
                settings,
                "WAGTAIL_WORDPRESS_IMPORTER_REQUESTS_SETTINGS",
                {
                    "headers": {"User-Agent": "WagtailWordpressImporter"},
                    "timeout": 5,
                    "stream": True,
                    "allow_redirects": allow_redirects,
                },
            ),
        )
        status = True if response.status_code == 200 else False
        return response, status, response.headers.get("content-type")
    except requests.ConnectionError:
        print(f"ConnectionError: {src}")
        return None, False, None
    except requests.HTTPError:
        print(f"HTTPError: {src}")
        return None, False, None
    except requests.RequestException:
        print(f"RequestException: {src}")
        return None, False, None
    except requests.ReadTimeout:
        print(f"ReadTimeout: {src}")
        return None, False, None
    except requests.Timeout:
        print(f"Timeout: {src}")
        return None, False, None
    except requests.ConnectTimeout:
        print(f"ConnectTimeout: {src}")
        return None, False, None


## FUNCTIONS FOR IMAGES


def image_linker(html):
    """
    params
    ======
        html: html from a single rich_text block

    returns
    =======
        string: the html with img tags modified

    BS4 performs a find and replace on all img tags found in the HTML.
    If the image can be retrieved from the remote site and saved into a Wagtail ImageModel
    the soup is modified.
    """
    soup = BeautifulSoup(html, "html.parser")
    images = soup.find_all("img")
    for image in images:
        if image.attrs and image.attrs.get("src"):
            image_src = get_absolute_src(
                image.attrs["src"],
                getattr(settings, "WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN"),
            )
            saved_image = get_or_save_image(image_src)
            if saved_image:
                image_embed = soup.new_tag("embed")
                image_embed.attrs["embedtype"] = "image"
                image_embed.attrs["id"] = saved_image.id
                image_embed.attrs["alt"] = get_image_alt(image)
                image_embed.attrs["format"] = get_alignment_class(image)
                image.replace_with(image_embed)
        else:
            print(f"IMAGE HAS NO SRC: {image}")

    return str(soup)


def get_or_save_image(src):
    image_file_name = get_image_file_name(src)
    existing_image = image_exists(image_file_name)
    if not existing_image:
        response, valid, type = fetch_url(src)
        if valid and (
            type
            in getattr(
                settings,
                "WAGTAIL_WORDPRESS_IMPORTER_VALID_IMAGE_CONTENT_TYPES",
                [
                    "image/gif",
                    "image/jpeg",
                    "image/png",
                    "image/webp",
                ],
            )
        ):
            temp_image = NamedTemporaryFile(delete=True)
            temp_image.name = image_file_name
            temp_image.write(response.content)
            temp_image.flush()
            retrieved_image = ImportedImage(
                file=File(file=temp_image), title=image_file_name
            )
            retrieved_image.save()
            temp_image.close()
            return retrieved_image
        else:
            print(f"RECEIVED INVALID IMAGE RESPONSE: {src}")
    return existing_image


## FUNCTIONS FOR DOCUMENTS


def document_linker(html):
    """
    params
    ======
        html: html from a single rich_text block

    returns
    =======
        string: the html with anchor links modified

    BS4 performs a find and replace on all img tags found in the HTML.
    If the image can be retrived from the remote site and saved into a Wagtail ImageModel
    the soup is modified.
    """
    soup = BeautifulSoup(html, "html.parser")
    anchors = soup.find_all("a")
    for anchor in anchors:
        if anchor.attrs and anchor.attrs.get("href"):
            anchor_href = get_absolute_src(
                anchor.attrs["href"],
                getattr(settings, "WAGTAIL_WORDPRESS_IMPORTER_SOURCE_DOMAIN"),
            )
            anchor_inner_content = anchor.text
            saved_document = get_or_save_document(anchor_href)
            if saved_document:
                document_embed = soup.new_tag("a")
                document_embed.attrs["linktype"] = "document"
                document_embed.attrs["id"] = saved_document.id
                document_embed.string = anchor_inner_content
                anchor.replace_with(document_embed)
        else:
            print(f"DOCUMENT HAS NO HREF: {anchor}")

    return str(soup)


def get_or_save_document(href):
    file_type = href.split(".")[-1]
    if file_type in getattr(
        settings,
        "",
        [
            "pdf",
            "ppt",
            "docx",
        ],
    ):
        document_file_name = get_document_file_name(href)
        existing_document = document_exists(document_file_name)
        if not existing_document:
            response, valid, type = fetch_url(href)
            if valid and (
                type
                in getattr(
                    settings,
                    "",
                    [
                        "application/pdf",
                        "application/vnd.openxmlformats-officedocument.presentationml.presentation",
                        "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                    ],
                )
            ):
                temp_document = NamedTemporaryFile(delete=True)
                temp_document.name = document_file_name
                temp_document.write(response.content)
                temp_document.flush()
                retrieved_document = ImportedDocument(
                    file=File(file=temp_document), title=document_file_name
                )
                retrieved_document.save()
                temp_document.close()
                return retrieved_document
            else:
                print(f"RECEIVED INVALID DOCUMENT RESPONSE: {href}")
        return existing_document


## STREAMFIELD BLOCKS


def build_block_quote_block(tag):
    block_dict = {
        "type": "block_quote",
        "value": {"quote": tag.text.strip(), "attribution": tag.cite},
    }
    return block_dict


def build_form_block(tag):
    block_dict = {"type": "raw_html", "value": str(tag)}
    return block_dict


def build_heading_block(tag):
    block_dict = {
        "type": "heading",
        "value": {"importance": tag.name, "text": tag.text},
    }
    return block_dict


def build_iframe_block(tag):
    block_dict = {
        "type": "raw_html",
        "value": '<div class="core-custom"><div class="responsive-iframe">{}</div></div>'.format(
            str(tag)
        ),
    }
    return block_dict


def build_image_block(tag):
    def get_image_id(src):
        return 1

    block_dict = {"type": "image", "value": get_image_id(tag.src)}
    return block_dict


def build_table_block(tag):
    block_dict = {"type": "raw_html", "value": str(tag)}
    return block_dict


## FALLBACK STREAMFIELD BLOCKS


def conf_fallback_block():
    return getattr(
        settings,
        "WAGTAIL_WORDPRESS_IMPORTER_FALLBACK_BLOCK",
        "wagtail_wordpress_import.block_builder_defaults.build_richtext_block_content",
    )


def build_richtext_block_content(html, blocks):
    """
    image_linker is called to link up and retrive the remote image
    document_linker is called to link up and retrive the remote documents
    filters are called to replace inline shortcodes
    """
    html = image_linker(html)
    html = document_linker(html)
    for inline_shortcode_handler in getattr(
        settings, "WAGTAIL_WORDPRESS_IMPORTER_INLINE_SHORTCODE_HANDLERS", []
    ):
        function = import_string(inline_shortcode_handler).construct_html_tag
        html = function(html)
    blocks.append({"type": "rich_text", "value": html})
    html = ""
    return html


## RICH TEXT FUNCTIONS


def get_image_alt(img_tag):
    return img_tag.attrs["alt"] if "alt" in img_tag.attrs else None


def get_image_file_name(src):
    return src.split("/")[-1] if src else None  # need the last part


def get_document_file_name(src):
    return src.split("/")[-1] if src else None  # need the last part


def image_exists(name):
    try:
        return ImportedImage.objects.get(title=name)
    except ImportedImage.DoesNotExist:
        pass


def document_exists(name):
    try:
        return ImportedDocument.objects.get(title=name)
    except ImportedDocument.DoesNotExist:
        pass


## GENERAL FUNCTIONS


def get_absolute_src(src, domain_prefix=None):
    src = src.lstrip("/")
    if not src.startswith("http") and domain_prefix:
        return domain_prefix + "/" + src
    return src


def get_alignment_class(image):
    alignment = "fullwidth"

    if "class" in image.attrs:
        if "align-left" in image.attrs["class"]:
            alignment = "left"
        elif "align-right" in image.attrs["class"]:
            alignment = "right"

    return alignment



import re


class InlineShortcodeHandler:
    """Subclasses should declare a shortcode_name and provide
    a construct_html_tag method for converting the shortcode to a HTML tag.
    """

    inline_shortcode_name: str

    element_name = "span"

    def __init__(self):
        # Subclasses should declare a shortcode_name
        if not hasattr(self, "shortcode_name"):
            raise NotImplementedError(
                "Create a subclass of BlockShortcodeHandler with a shortcode_name attribute"
            )
        pattern = re.compile(r"^\S[a-zA-Z0-9_\S]+\S$")
        # shortcode_name must use upper or lower case letters or digits and cannot contain spaces
        if not pattern.match(self.shortcode_name):
            raise ValueError(
                "The shortcode_name attribute must use upper or lower case letters or digits and cannot contain spaces"
            )

    @property
    def _pattern(self):
        """Return a regex to match an inline shortcode and capture the attrs.

        Given an input:

            "Preface [foo bar=1] epilogue."

        the regex will match the string between the "[" and "]"  tags. The
        capture group "attrs" will match " bar=1", and capture group "shortcodename" will
        match "foo".
        \[(\S+)(?:\s)(\w\S.+)\]
        """

        return re.compile(
            r"\["  # matches the opening [
            + self.shortcode_name
            + r"(?:\s)"  # matches a single space
            + r"(?P<attrs>[^\]]*)"  # capture 'attrs', matching anything but ]
            + r"\]"  # matches the closing ] of the opening tag
        )

    @staticmethod
    def get_shortcode_attrs(string):
        """Create and return a dict of the attrs of a shortcode."""
        attrs = string.split(" ")
        attrs = [attr.split("=") for attr in attrs]
        return {attr[0]: attr[1].replace('"', "") for attr in attrs}
